//////////////////////////////////////////////////////////////////////////////
// File name    : tievm_bmc.v
// Project      : TI_C6457_LCEVM 
// Created On   : 05/08/2010 (DD/MM/YYYY) 
// Developers   : Christian Felix Y.
// Description  : Boot Mode controller module is responsible for providing the
//              : boot-up sequence to the TI_C6457 DSP. This module provides
//                the data to the GPIO lines via GPIO interface controller 
//                module during cold(power-on) reset or warm reset. The module
//                also drives POR# and RESET# to the DSP during cold and warm
//                reset respectively.
// NOTES        :
// Limitations  :
// Known Errors :
// Revision     : $Log: tievm_bmc.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.4  2010/09/16 08:04:45  christianfelix_rw
// Revision     : New Boot Sequence Implemented.
// Revision     :
// Revision     : Revision 1.3  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.2  2010/09/03 10:27:31  christianfelix_rw
// Revision     : Reset count done flag set to 0 by default.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:01:54  christianfelix_rw
// Revision     : Initial Version
// Revision     :
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module tievm_bmc (
                  //Clock & Reset Inputs
                  tievm_fpga_clk_i            ,
                  tievm_sys_rst_ni            ,
                  tievm_warm_rst_i            ,

                  // PLL signal
                  tievm_pll_lock_i            ,

                  // DIP and User Switches

                  tievm_dip_sw1_lendian_i     ,
                  tievm_dip_sw2_btmode0_i     ,
                  tievm_dip_sw3_btmode1_i     ,
                  tievm_dip_sw4_btmode2_i     ,
                  tievm_dip_sw5_btmode3_i     ,
                  tievm_dip_sw6_devnum0_i     ,
                  tievm_dip_sw7_devnum1_i     ,
                  tievm_dip_sw8_usrsw1_i      ,
                  tievm_dip_sw9_usrsw2_i      ,
                  tievm_dip_sw10_cfgp0_i      ,
                  tievm_dip_sw11_cfgp1_i      ,
                  tievm_dip_sw12_cfgp2_i      ,
                  tievm_dip_sw13_cfgp3_i      ,
                  tievm_dip_sw14_cfgp4_i      ,
                  tievm_dip_sw15_hpiwidth_i   ,
                  tievm_dip_sw16_eclkinsel_i  ,

                  // TI DSP signal inputs

                  tievm_dsp_fpga_rst_status_i ,

                  // LED inputs
                  tievm_dsp_bmc_led0_i        ,
                  tievm_dsp_bmc_led1_i        ,


                  // LED outputs
                  tievm_user_led1_o           ,
                  tievm_user_led2_o           ,

                  // TI DSP signal outputs
                  tievm_bmc_por_no            ,
                  tievm_bmc_reset_no          ,

                  //GIC outputs
                  tievm_bmc_gic_boot_done_o   ,
                  tievm_bmc_gic_data_o                       
                 );

///////////////////////////////////////////////////////////////////////////////
// Parameters Declaration
///////////////////////////////////////////////////////////////////////////////

parameter POR_COUNT                  = 23'h7A19D0 ; // 50us+200ms
parameter RESET_COUNT                = 12'h7D0    ; // 50us
parameter EMAC_BOOT_POR2RESET_DELAY  = 28'h9896FD0; // 4s

///////////////////////////////////////////////////////////////////////////////
// Use the below mentioned parameters for SIMULATION ONLY. 
// Reason is that - The above mentioned parameters 22'h3D10D0 and 28'h9896FD0 
//                  are very large as they count for 100.05ms and 4sec resp.
//                  So, single test-case will take quite a long time and 
//                  Running Regression would be Impossible.
// While performing SIMULATION please use the parameters mentioned below and
// comment the above parameters with value 22'h3D10D0 and 28'h9896FD0.
//
// --> parameter POR_COUNT                 = 23'h1F40; (Can Take any      
// --> parameter EMAC_BOOT_POR2RESET_DELAY = 28'h1388;  SMALL VALUE)
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Inputs ports declaration
///////////////////////////////////////////////////////////////////////////////

// Clock & Reset Inputs
input         tievm_fpga_clk_i            ;
input         tievm_sys_rst_ni            ;
input         tievm_warm_rst_i            ;

// PLL signal
input         tievm_pll_lock_i            ;

// User and DIP switches
input         tievm_dip_sw1_lendian_i     ;
input         tievm_dip_sw2_btmode0_i     ;
input         tievm_dip_sw3_btmode1_i     ;
input         tievm_dip_sw4_btmode2_i     ;
input         tievm_dip_sw5_btmode3_i     ;
input         tievm_dip_sw6_devnum0_i     ;
input         tievm_dip_sw7_devnum1_i     ;
input         tievm_dip_sw8_usrsw1_i      ;
input         tievm_dip_sw9_usrsw2_i      ;
input         tievm_dip_sw10_cfgp0_i      ;
input         tievm_dip_sw11_cfgp1_i      ;
input         tievm_dip_sw12_cfgp2_i      ;
input         tievm_dip_sw13_cfgp3_i      ;
input         tievm_dip_sw14_cfgp4_i      ;
input         tievm_dip_sw15_hpiwidth_i   ;
input         tievm_dip_sw16_eclkinsel_i  ;

// TI DSP signal inputs
input         tievm_dsp_fpga_rst_status_i ;

// LED inputs
input         tievm_dsp_bmc_led0_i        ;                            
input         tievm_dsp_bmc_led1_i        ;                            

////////////////////////////////////////////////////////////////////////////////
// Outputs ports declaration
////////////////////////////////////////////////////////////////////////////////

// LED outputs
output        tievm_user_led1_o           ;
output        tievm_user_led2_o           ;

// TI DSP signal outputs
output        tievm_bmc_por_no            ;
output        tievm_bmc_reset_no          ;

//GIC outputs
output        tievm_bmc_gic_boot_done_o   ;
output [15:0] tievm_bmc_gic_data_o        ;

////////////////////////////////////////////////////////////////////////////////
// Inputs ports wire declaration
////////////////////////////////////////////////////////////////////////////////


//Clock & Reset Inputs
wire          tievm_fpga_clk_i            ;
wire          tievm_sys_rst_ni            ;
wire          tievm_warm_rst_i            ;

// PLL signal
wire          tievm_pll_lock_i            ;

// DIP switches
wire          tievm_dip_sw1_lendian_i     ;
wire          tievm_dip_sw2_btmode0_i     ;
wire          tievm_dip_sw3_btmode1_i     ;
wire          tievm_dip_sw4_btmode2_i     ;
wire          tievm_dip_sw5_btmode3_i     ;
wire          tievm_dip_sw6_devnum0_i     ;
wire          tievm_dip_sw7_devnum1_i     ;
wire          tievm_dip_sw8_usrsw1_i      ;
wire          tievm_dip_sw9_usrsw2_i      ;
wire          tievm_dip_sw10_cfgp0_i      ;
wire          tievm_dip_sw11_cfgp1_i      ;
wire          tievm_dip_sw12_cfgp2_i      ;
wire          tievm_dip_sw13_cfgp3_i      ;
wire          tievm_dip_sw14_cfgp4_i      ;
wire          tievm_dip_sw15_hpiwidth_i   ;
wire          tievm_dip_sw16_eclkinsel_i  ;

// TI DSP signal inputs
wire          tievm_dsp_fpga_rst_status_i ;

////////////////////////////////////////////////////////////////////////////////
// Outputs ports wire/reg declaration
////////////////////////////////////////////////////////////////////////////////

// LED outputs
wire          tievm_user_led1_o           ;
wire          tievm_user_led2_o           ;

// TI DSP signal outputs
reg           tievm_bmc_por_no            ;
reg           tievm_bmc_reset_no          ;

//GIC outputs
reg           tievm_bmc_gic_boot_done_o   ;
reg    [15:0] tievm_bmc_gic_data_o        ;

////////////////////////////////////////////////////////////////////////////////
// Local wire/reg declaration
////////////////////////////////////////////////////////////////////////////////

reg    [22:0] por_cnt                     ;
reg    [11:0] reset_cnt                   ;
reg    [27:0] emac_boot_por2rst_cnt       ;
reg           internal_warm_rst_d         ;
reg           internal_warm_rst_dd        ;
reg           tievm_pll_lock_d            ;
reg           reset_stat_d                ;
reg           por_cnt_done                ;
reg           reset_cnt_done              ;

wire          tievm_bmc_warm_rst          ;
wire          internal_warm_rst_w         ;

wire          por_cnt_over                ;
wire          reset_cnt_over              ;
wire          delay_cnt_over              ;
reg           any_reset                   ;
wire          pll_lock_edge               ;
wire          reset_stat_edge             ;
wire   [15:0] gic_switch_data             ;
reg           tievm_bmc_por_no_d          ;
reg           tievm_bmc_reset_no_d        ;
reg           delay_counter_en            ;
reg           emac_boot_detected          ;

////////////////////////////////////////////////////////////////////////////////
// Continuous Assignment Statements
////////////////////////////////////////////////////////////////////////////////

assign tievm_bmc_warm_rst = tievm_warm_rst_i | internal_warm_rst_w;

assign por_cnt_over       = (por_cnt   == POR_COUNT  );
assign reset_cnt_over     = (reset_cnt == RESET_COUNT);
assign delay_cnt_over     = (emac_boot_por2rst_cnt == EMAC_BOOT_POR2RESET_DELAY);

assign pll_lock_edge     = ((tievm_pll_lock_i           ) & (!tievm_pll_lock_d));
assign reset_stat_edge   = ((tievm_dsp_fpga_rst_status_i) & (!reset_stat_d    ));

assign gic_switch_data   = {tievm_dip_sw16_eclkinsel_i, tievm_dip_sw15_hpiwidth_i,
                            tievm_dip_sw14_cfgp4_i    , tievm_dip_sw13_cfgp3_i   ,
                            tievm_dip_sw12_cfgp2_i    , tievm_dip_sw11_cfgp1_i   ,
                            tievm_dip_sw10_cfgp0_i    , tievm_dip_sw9_usrsw2_i   ,
                            tievm_dip_sw8_usrsw1_i    , tievm_dip_sw7_devnum1_i  ,
                            tievm_dip_sw6_devnum0_i   , tievm_dip_sw5_btmode3_i  ,
                            tievm_dip_sw4_btmode2_i   , tievm_dip_sw3_btmode1_i  ,
                            tievm_dip_sw2_btmode0_i   , tievm_dip_sw1_lendian_i };

assign tievm_user_led1_o = tievm_dsp_bmc_led0_i;
assign tievm_user_led2_o = tievm_dsp_bmc_led1_i;

///////////////////////////////////////////////////////////////////////////////
// Logic for Detecting EMAC BOOT OPERATION
///////////////////////////////////////////////////////////////////////////////

always @(*)
  begin
     case({tievm_dip_sw5_btmode3_i,tievm_dip_sw4_btmode2_i,tievm_dip_sw3_btmode1_i,tievm_dip_sw2_btmode0_i})

     4'b0110 : emac_boot_detected = 1'b1; 
     4'b0111 : emac_boot_detected = 1'b1; 
     4'b1000 : emac_boot_detected = 1'b1; 
     default : emac_boot_detected = 1'b0;

     endcase
  end

///////////////////////////////////////////////////////////////////////////////
// Logic for Asserting Warm Reset as well as Cold Reset to the DSP upon
// receiving Cold Reset Input
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_fpga_clk_i or negedge tievm_sys_rst_ni)
  begin
     if(!tievm_sys_rst_ni)
       begin
          delay_counter_en <= 1'b0;
       end
     else if(delay_cnt_over)
       begin
          delay_counter_en <= 1'b0;
       end
     else if(por_cnt_over && emac_boot_detected)
       begin
          delay_counter_en <= 1'b1;
       end
     else
       begin
          delay_counter_en <= delay_counter_en;
       end
  end

////////////////////////////////////////////////////////////////////////////////
// Delay Counter for Assering Warm Reset after 4s if EMAC Boot Mode is detected
////////////////////////////////////////////////////////////////////////////////

always@(posedge tievm_fpga_clk_i or negedge tievm_sys_rst_ni)
  begin
     if(!tievm_sys_rst_ni)
       begin
          emac_boot_por2rst_cnt <= 'h0;
       end
     else if(tievm_pll_lock_i && delay_counter_en)
       begin
          emac_boot_por2rst_cnt <= emac_boot_por2rst_cnt + 1'b1;
       end
     else
       begin
          emac_boot_por2rst_cnt <= 'h0;
       end
  end

///////////////////////////////////////////////////////////////////////////////
// Logic for generating internal warm reset
///////////////////////////////////////////////////////////////////////////////

assign internal_warm_rst_w = (delay_cnt_over | internal_warm_rst_d);

always @(posedge tievm_fpga_clk_i or negedge tievm_sys_rst_ni)
  begin
     if(!tievm_sys_rst_ni)
       begin
          internal_warm_rst_d  <= 1'b0;
       end
     else
       begin
          internal_warm_rst_d  <= delay_cnt_over;
       end
  end

///////////////////////////////////////////////////////////////////////////////
// Logic for common reset from warm and cold reset 
///////////////////////////////////////////////////////////////////////////////

always@(*)
  begin
     if(!tievm_sys_rst_ni)
       begin
          any_reset =  1'b0;
       end
     else if(tievm_warm_rst_i)
       begin
          any_reset =  1'b0;
       end
     else
       begin
          any_reset =  1'b1;
       end
  end

///////////////////////////////////////////////////////////////////////////////
// Sequential Assignments
///////////////////////////////////////////////////////////////////////////////

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          tievm_pll_lock_d <= 'b0;
          reset_stat_d     <= 'b0;
       end
     else
       begin
          tievm_pll_lock_d <= tievm_pll_lock_i           ;
          reset_stat_d     <= tievm_dsp_fpga_rst_status_i;
       end
  end

////////////////////////////////////////////////////////////////////////////////
// POR# signal generation logic 
////////////////////////////////////////////////////////////////////////////////

always@(*)
  begin
     if(!tievm_sys_rst_ni)
       begin
          tievm_bmc_por_no = 1'b0;
       end
     else if(tievm_pll_lock_i && por_cnt_done)
       begin
          tievm_bmc_por_no = 1'b1;
       end
     else
       begin
          tievm_bmc_por_no = tievm_bmc_por_no_d;
       end
  end

// POR# signal counter

always@(posedge tievm_fpga_clk_i or negedge tievm_sys_rst_ni)
  begin
     if(!tievm_sys_rst_ni)
       begin
          por_cnt <= 10'd0;
       end
     else if(tievm_pll_lock_i && !por_cnt_done)
       begin
          por_cnt <= por_cnt + 1'b1;
       end
     else
       begin
          por_cnt <= 'b0;
       end
  end

always@(posedge tievm_fpga_clk_i or negedge tievm_sys_rst_ni)
  begin
     if(!tievm_sys_rst_ni)
       begin
          por_cnt_done = 1'b0;
       end
     else if(tievm_pll_lock_i && por_cnt_over)
       begin
          por_cnt_done = 1'b1;
       end     
     else
       begin
          por_cnt_done = por_cnt_done;
       end     
  end

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          tievm_bmc_por_no_d <=  tievm_sys_rst_ni;
       end
     else
       begin
          tievm_bmc_por_no_d <=  tievm_bmc_por_no;
       end
  end  

////////////////////////////////////////////////////////////////////////////////
// RESET# signal generation logic
////////////////////////////////////////////////////////////////////////////////

always@(*)
  begin
     if(tievm_bmc_warm_rst | !tievm_sys_rst_ni)
       begin
          tievm_bmc_reset_no = 1'b0;
       end
     else if(tievm_pll_lock_i && reset_cnt_done)
       begin
          tievm_bmc_reset_no = 1'b1;
       end
     else
       begin
          tievm_bmc_reset_no = tievm_bmc_reset_no_d;
       end
  end

// RESET# signal counter

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          reset_cnt <= 'b0;
       end
     else 
       begin
          if(tievm_bmc_warm_rst)
            begin
               reset_cnt <= 'b0;
            end
          else if (tievm_pll_lock_i && !reset_cnt_done)
            begin
               reset_cnt <= reset_cnt + 1'b1;
            end
          else
            begin
               reset_cnt <= 'b0;
            end
       end
  end

always@(*)
  begin
     if(tievm_warm_rst_i | internal_warm_rst_w)
       begin
          reset_cnt_done = 1'b0;
       end
     else if (tievm_pll_lock_i && !reset_cnt_over)
       begin
          reset_cnt_done = 1'b0;
       end
     else if (tievm_pll_lock_i &&  reset_cnt_over)
       begin
          reset_cnt_done = 1'b1;
       end
     else 
       begin
          reset_cnt_done = 1'b0;
       end
  end   

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          tievm_bmc_reset_no_d <= 1'b0;
       end
     else
       begin
          tievm_bmc_reset_no_d <= tievm_bmc_reset_no;
       end
  end 

////////////////////////////////////////////////////////////////////////////////
// GPIO signal generation
////////////////////////////////////////////////////////////////////////////////

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          tievm_bmc_gic_boot_done_o <= 'b0;
       end
     else if (tievm_pll_lock_i & reset_stat_edge)
       begin
          tievm_bmc_gic_boot_done_o <= 'b1;
       end
     else
       begin
          tievm_bmc_gic_boot_done_o <= tievm_bmc_gic_boot_done_o;
       end
  end 

always@(posedge tievm_fpga_clk_i or negedge any_reset)
  begin
     if(!any_reset)
       begin
          tievm_bmc_gic_data_o  <= 'h0;
       end
     else
       begin
          if(tievm_pll_lock_i)
            begin
              tievm_bmc_gic_data_o <= gic_switch_data;
            end
          else
            begin
              tievm_bmc_gic_data_o <= 'h0;
            end
       end
  end  

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

