////////////////////////////////////////////////////////////////////////////////
// File name    : tievm_cntr.v
// Project      : TI_C6457_LCEVM
// Created On   : 05/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This is a parameterized up counter with sync/async reset
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_cntr.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps
module tievm_cntr
       (
        clk_i      ,                    // Clock Input
        rst_n_i    ,                    // Async Reset Input
        sc_i ,                          // SYnc Reset Input
        count_o,                        // Counter Output
        count_up_o
        );
//*****************************************************************************
//                          PARAMETER DECLARATIONS
//*****************************************************************************
parameter CNTR_WIDTH = 5;
parameter UP_COUNT = 5'd2;

//*****************************************************************************
//                          INPUT PORT DECLARATIONS
//*****************************************************************************

input                                   clk_i      ;
input                                   rst_n_i    ;
input                                   sc_i ;

//*****************************************************************************
//                          OUTPUT PORT DECLARATIONS
//*****************************************************************************

output [CNTR_WIDTH - 1 : 0]             count_o    ;
output                                  count_up_o;

//*****************************************************************************
//                          OUTPUT REGISTER DECLARATIONS
//*****************************************************************************

reg    [CNTR_WIDTH - 1 : 0]             count_o    ;
reg                                     count_up_o;


//*****************************************************************************
//                          INTERNAL REGISTER DECLARATIONS
//*****************************************************************************

//*****************************************************************************
//                          INTERNAL WIRE DECLARATIONS
//*****************************************************************************


//Start of the code
//
//*****************************************************************************
//The Following Sequential Block implements a Generic Up Counter. 
//The Counter is reset to all '0' when it gets a Async Reset from the TOP
//Block.
//The Counter also has a Sync Reset. When Asserted, the Sync Reset
//Synchronously Resets the Counter to all '0's.
//The Counter Increments on the Every Clock cycle of Clock input By '1' 
//*****************************************************************************
//

always @(count_o)
begin
  if(count_o == UP_COUNT)
  begin
    count_up_o = 1'b1;
  end
  else
  begin
    count_up_o = 1'b0;
  end
end

always @ (posedge clk_i or negedge rst_n_i)
begin
  if (~rst_n_i)
  begin
    count_o <= {CNTR_WIDTH{1'b0}};  
  end // if (~rst_n_i)
  else
  begin
    if (sc_i)
    begin
      count_o <= {CNTR_WIDTH{1'b0}} ; 
    end // if (sc_i)
    else 
    begin
      count_o <= count_o + 1'b1     ;
    end // else (~sc_i)
  end // else (~rst_i)
end // always @ (pos.....)

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

