//////////////////////////////////File Header//////////////////////////////////
// Module Name : tievm_fpga_sys_reset_sync  
// Originator  : Kaushal D. Buch 
// Date        : 07/28/2009 
// Description : Parameterized clock synchronizer module with reset.
//							          
// Revision    : $Log: tievm_fpga_reset_sync.v,v $
// Revision    : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision    : added
// Revision    :
// Revision    : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision    : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision    :
// Revision    : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision    : Copyright Added.
// Revision    :
// Revision    : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision    : Initial Version
// Revision    :
// Open Issues : None
//////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module tievm_fpga_reset_sync ( 
                     sys_clk,
                     rst,
                     rst_sync
                    );

//////////////////////////////////////
// Input/Output
//////////////////////////////////////

input               sys_clk ;   // Input CIF clock
input               rst ;       // Input reset
output              rst_sync ;  // Synchronized reset output with respect to sys_clk

//////////////////////////////////////
// Input/Output declaration
//////////////////////////////////////

wire               sys_clk ;
wire               rst ;
wire               rst_sync ;

//////////////////////////////////////
// Local declaration                    
//////////////////////////////////////

reg                out_reg1 ;
reg                out_reg2 ;

//////////////////////////////////////
// Clock synch logic -SYS Clock                    
//////////////////////////////////////
  always @ (posedge sys_clk or negedge rst)
   begin
     
     if (!rst)
     begin
     out_reg1 <= 1'b0    ;
     out_reg2 <= 1'b0    ;
     end
     
     else
     begin
     out_reg1 <= 1'b1     ;
     out_reg2 <= out_reg1 ;
     end
 
  end 

//////////////////////////////////////
// Generating out 
//////////////////////////////////////
  assign rst_sync = out_reg2  ;

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

