////////////////////////////////////////////////////////////////////////////////////
// File name    : tievm_fpga_top.v
// Project      : TI_C6457_LCEVM
// Created On   : 08/06/2010 (MM/DD/YYYY)
// Developers   : Christian Felix Y.
// Description  : This is the Top Module for TI_C6457_LCEVM FPGA
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_fpga_top.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.3  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.2  2010/09/03 10:30:37  christianfelix_rw
// Revision     : Removed Unnecessary Commented Code.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////
 
`timescale 1ns/1ps 

module tievm_fpga_top (
                       //Clock & Reset Inputs
                       tievm_fpga_clk_i            , // 25MHz Clock Input
                       tievm_warm_rst_ni           , // Warm Reset
                       tievm_crm_gic_rst_ni        , // Cold Reset
                       
                       // DIP and User Switches
                       tievm_dip_sw1_lendian_i     ,
                       tievm_dip_sw2_btmode0_i     ,
                       tievm_dip_sw3_btmode1_i     ,
                       tievm_dip_sw4_btmode2_i     ,
                       tievm_dip_sw5_btmode3_i     ,
                       tievm_dip_sw6_devnum0_i     ,
                       tievm_dip_sw7_devnum1_i     ,
                       tievm_dip_sw8_usrsw1_i      ,
                       tievm_dip_sw9_usrsw2_i      ,
                       tievm_dip_sw10_cfgp0_i      ,
                       tievm_dip_sw11_cfgp1_i      ,
                       tievm_dip_sw12_cfgp2_i      ,
                       tievm_dip_sw13_cfgp3_i      ,
                       tievm_dip_sw14_cfgp4_i      ,
                       tievm_dip_sw15_hpiwidth_i   ,
                       tievm_dip_sw16_eclkinsel_i  ,
                       tievm_bridge_intrpt_ni      ,
                                                                     
                       //Other signals for control from DSP to FPGA
                       tievm_dsp_fpga_rst_status_i ,
                       tievm_bmc_por_no            ,
                       tievm_bmc_reset_no          ,
                                                                     
                       //FPGA-NAND Flash Interface signals
                       nfm_nfc_io                  ,
                       nfc_nfm_ce_no               ,
                       nfc_nfm_cle_o               ,
                       nfc_nfm_ale_o               ,
                       nfc_nfm_re_no               ,
                       nfc_nfm_we_no               ,
                       nfc_nfm_wp_no               ,
                       nfm_nfc_rb_i                ,
                                                                  
                       // LED outputs
                       tievm_user_led1_o           ,
                       tievm_user_led2_o           ,
                                                                     
                       // Dual purpose DSP GPIO to FPGA Interface
                       tievm_dsp_gic_data_io       ,
                       tievm_dsp_gic_cmd_io        ,
                       tievm_dsp_gic_stb_io        ,
                       tievm_gic_dsp_rdy_o         , 
                       tievm_dsp_gic_mode_sel_io   ,
                       tievm_fpga_dsp_uart_intr_o  ,
                       tievm_dsp_fpga_led0_io      ,
                       tievm_dsp_fpga_led1_io      ,
                       tievm_fpga_dsp_nmi_no       ,
                       tievm_fpga_prbs_o           ,

                       // Other signals
                       tievm_amc_detect_ni         ,
                       tievm_board_ver_i
                      );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////

parameter TIEVM_GPIO_BUS_WIDTH =  8;
parameter TIEVM_GPIO_CMD_WIDTH =  2;

///////////////////////////////////////////////////////////////////////////////
// Input port declaration
///////////////////////////////////////////////////////////////////////////////

input                                 tievm_fpga_clk_i                ;
input                                 tievm_warm_rst_ni               ;
input                                 tievm_crm_gic_rst_ni            ; 
input                                 tievm_dip_sw1_lendian_i         ;
input                                 tievm_dip_sw2_btmode0_i         ;
input                                 tievm_dip_sw3_btmode1_i         ;
input                                 tievm_dip_sw4_btmode2_i         ;
input                                 tievm_dip_sw5_btmode3_i         ;
input                                 tievm_dip_sw6_devnum0_i         ;
input                                 tievm_dip_sw7_devnum1_i         ;
input                                 tievm_dip_sw8_usrsw1_i          ;
input                                 tievm_dip_sw9_usrsw2_i          ;
input                                 tievm_dip_sw10_cfgp0_i          ;
input                                 tievm_dip_sw11_cfgp1_i          ;
input                                 tievm_dip_sw12_cfgp2_i          ;
input                                 tievm_dip_sw13_cfgp3_i          ;
input                                 tievm_dip_sw14_cfgp4_i          ;
input                                 tievm_dip_sw15_hpiwidth_i       ;
input                                 tievm_dip_sw16_eclkinsel_i      ;
input                                 tievm_bridge_intrpt_ni          ;
input                                 tievm_dsp_fpga_rst_status_i     ;
input                                 nfm_nfc_rb_i                    ;
input                                 tievm_amc_detect_ni             ;
input  [                         2:0] tievm_board_ver_i               ;

///////////////////////////////////////////////////////////////////////////////
// Output port declaration
///////////////////////////////////////////////////////////////////////////////

output                                tievm_bmc_por_no                ;
output                                tievm_bmc_reset_no              ;
output                                nfc_nfm_ce_no                   ;
output                                nfc_nfm_cle_o                   ;
output                                nfc_nfm_ale_o                   ;
output                                nfc_nfm_re_no                   ;
output                                nfc_nfm_we_no                   ;
output                                nfc_nfm_wp_no                   ;
output                                tievm_user_led1_o               ;
output                                tievm_user_led2_o               ;
output                                tievm_fpga_prbs_o               ;
output                                tievm_gic_dsp_rdy_o             ; 
output                                tievm_fpga_dsp_uart_intr_o      ;
output                                tievm_fpga_dsp_nmi_no           ;

///////////////////////////////////////////////////////////////////////////////
// Inout ports declaration
///////////////////////////////////////////////////////////////////////////////

inout  [TIEVM_GPIO_BUS_WIDTH - 1 : 0] tievm_dsp_gic_data_io           ;
inout  [TIEVM_GPIO_CMD_WIDTH - 1 : 0] tievm_dsp_gic_cmd_io            ;
inout                                 tievm_dsp_gic_stb_io            ; 
inout                                 tievm_dsp_fpga_led0_io          ; 
inout                                 tievm_dsp_fpga_led1_io          ; 
inout                                 tievm_dsp_gic_mode_sel_io       ;
inout  [(`MEM_DATA_WIDTH - 1)    : 0] nfm_nfc_io                      ;

///////////////////////////////////////////////////////////////////////////////
//Local wire/reg declaration
///////////////////////////////////////////////////////////////////////////////

wire [TIEVM_GPIO_CMD_WIDTH - 1   : 0] tievm_dsp_gic_cmd_i             ;         
wire                                  tievm_dsp_gic_stb_i             ;
wire                                  tievm_dsp_bmc_led0_i            ;
wire                                  tievm_dsp_bmc_led1_i            ;
wire                                  tievm_fpga_clk_i                ;
wire                                  tievm_crm_gic_rst_ni            ;
wire                                  tievm_warm_rst_ni               ;

// User switches
wire                                  tievm_user_sw1_w                ;
wire                                  tievm_user_sw2_w                ;

// DIP switches
wire                                  tievm_dip_sw1_lendian_i         ;
wire                                  tievm_dip_sw2_btmode0_i         ;
wire                                  tievm_dip_sw3_btmode1_i         ;
wire                                  tievm_dip_sw4_btmode2_i         ;
wire                                  tievm_dip_sw5_btmode3_i         ;
wire                                  tievm_dip_sw6_devnum0_i         ;
wire                                  tievm_dip_sw7_devnum1_i         ;
wire                                  tievm_dip_sw8_usrsw1_i          ;
wire                                  tievm_dip_sw9_usrsw2_i          ;
wire                                  tievm_dip_sw10_cfgp0_i          ;
wire                                  tievm_dip_sw11_cfgp1_i          ;
wire                                  tievm_dip_sw12_cfgp2_i          ;
wire                                  tievm_dip_sw13_cfgp3_i          ;
wire                                  tievm_dip_sw14_cfgp4_i          ;
wire                                  tievm_dip_sw15_hpiwidth_i       ;
wire                                  tievm_dip_sw16_eclkinsel_i      ;

// TI DSP signal inputs
wire                                  tievm_dsp_fpga_rst_status_i     ;
wire                                  tievm_fpga_prbs_o               ;
reg                                   tievm_dsp_fpga_pt_mode_en       ;
wire                                  nfm_nfc_rb_i                    ;

wire   [TIEVM_GPIO_BUS_WIDTH - 1 : 0] tievm_dsp_gic_data_io           ;
wire   [(`MEM_DATA_WIDTH - 1)    :0 ] nfm_nfc_io                      ;
wire                                  tievm_gic_dsp_rdy_w             ;   
wire                                  tievm_gic_dsp_rdy_o             ; 

// LED outputs
wire                                  tievm_user_led1_o               ;
wire                                  tievm_user_led2_o               ;

// TI DSP signal outputs
wire                                  tievm_bmc_por_no                ;
wire                                  tievm_bmc_reset_no              ;

// FPGA-NAND Flash Signals
wire                                  nfc_nfm_ce_no                   ;
wire                                  nfc_nfm_cle_o                   ;
wire                                  nfc_nfm_ale_o                   ;
wire                                  nfc_nfm_re_no                   ;
wire                                  nfc_nfm_we_no                   ;
wire                                  nfc_nfm_wp_no                   ;
wire                                  nfc_nfm_ben_o                   ;

wire   [                         7:0] tievm_dsp_gic_data_i            ;
wire   [                         7:0] tievm_gic_dsp_data_o            ;
wire   [                         7:0] tievm_gic_nfc_cwr_reg_h_o       ;   
wire   [                         7:0] tievm_gic_nfc_cwr_reg_l_o       ;   
wire   [                         7:0] tievm_gic_nfc_cmd_reg_h_o       ;   
wire   [                         7:0] tievm_gic_nfc_cmd_reg_l_o       ;   
wire   [                         7:0] tievm_gic_nfc_on_off_addr_h_o   ;
wire   [                         7:0] tievm_gic_nfc_on_off_addr_l_o   ;
wire   [                         7:0] tievm_gic_nfc_rd_timeout_h_o    ;
wire   [                         7:0] tievm_gic_nfc_rd_timeout_l_o    ;
wire   [                         7:0] tievm_gic_nfc_wr_timeout_h_o    ;
wire   [                         7:0] tievm_gic_nfc_wr_timeout_l_o    ;
wire   [                        15:0] tievm_gic_nfc_erase_timeout_h_o ;
wire   [                        15:0] tievm_gic_nfc_erase_timeout_l_o ;
wire   [                         7:0] nfm_nfc_o                       ;
wire                                  tievm_gic_top_outdata_en_o      ;
wire                                  tievm_bmc_gic_boot_done_o       ;
wire                                  tievm_gic_dsp_nfc_intr_w        ;
wire   [                         7:0] tievm_fpga_dsp_data_w           ;
wire   [                        15:0] tievm_bmc_top_data_w            ;
wire   [                        31:0] tievm_gic_nfc_nfm_addr_reg_o    ;
wire   [                         7:0] tievm_gic_nfc_wdata_o           ;
wire   [                        31:0] nfc_sys_addr_i                  ;
wire   [                         7:0] tievm_user_sw_w                 ;
wire                                  user_led_en_w                   ;
wire                                  tievm_user_led1_w               ;
wire                                  tievm_user_led0_w               ;
wire                                  tievm_nand_wp_w                 ;
wire                                  tievm_fpga_prbs_w               ;
wire                                  warm_rst_sync_clk40_w           ;
wire                                  cold_rst_sync_clk40_w           ;
wire                                  sys_rst_sync_clk40_w            ;
wire                                  warm_rst_sync_clk80_w           ;
wire                                  cold_rst_sync_clk80_w           ;
wire                                  sys_rst_sync_clk80_w            ;

// Signals for Pass Through Mode

wire   [                         7:0] tievm_dsp_gic_data_w            ;
wire   [                         1:0] tievm_dsp_gic_cmd_w             ;
wire                                  tievm_gic_dsp_ready_w           ;
wire                                  tievm_dsp_gic_stb_w             ;
wire                                  tievm_dsp_gic_mode_sel_i        ;
wire                                  tievm_dsp_fpga_led0_i           ;
wire                                  tievm_dsp_fpga_led1_i           ;
wire                                  ptm_rd_en_nfc                   ;
reg                                   ptm_rd_en_nfc_d1                ;
reg                                   ptm_rd_en_nfc_d2                ;
wire                                  tievm_40MHz_clk                 ;
wire                                  tievm_80MHz_clk                 ;
wire                                  pll_lock_o                      ;
wire                                  tievm_10MHz_clk                 ;        
wire                                  tievm_prbs_10KHz_clk            ;
reg    [                         8:0] prand_number_gen                ;
wire                                  tievm_gic_nfc_ecc_mode_sel_w    ;
wire   [                         7:0] nfm_nfc_i                       ;
reg                                   tievm_amc_detect                ;
reg    [                         2:0] tievm_board_ver                 ;

///////////////////////////////////////////////////////////////////////////////
// Assignments for DSP GPIO-FPGA Interface
///////////////////////////////////////////////////////////////////////////////

assign tievm_fpga_dsp_nmi_no = 1'b0;
assign ptm_rd_en_nfc         = (tievm_dsp_fpga_led0_i & ptm_rd_en_nfc_d1 & ptm_rd_en_nfc_d2);

always @(posedge tievm_80MHz_clk or negedge sys_rst_sync_clk80_w)
  begin
     if(!sys_rst_sync_clk80_w)
       begin
          ptm_rd_en_nfc_d1 <= 1'b0;
          ptm_rd_en_nfc_d2 <= 1'b0;
       end
     else
       begin
          ptm_rd_en_nfc_d1 <= tievm_dsp_fpga_led0_i;
          ptm_rd_en_nfc_d2 <= ptm_rd_en_nfc_d1     ;
       end
  end

assign nfm_nfc_io    = (tievm_dsp_fpga_pt_mode_en && ptm_rd_en_nfc) ? tievm_dsp_gic_data_w :
                       {`MEM_DATA_WIDTH{1'bz}    };
assign nfc_nfm_ce_no = (tievm_dsp_fpga_pt_mode_en) ? tievm_dsp_fpga_led1_i  : 1'b1;
assign nfc_nfm_cle_o = (tievm_dsp_fpga_pt_mode_en) ? tievm_dsp_gic_cmd_w[0] : 1'b0; 
assign nfc_nfm_ale_o = (tievm_dsp_fpga_pt_mode_en) ? tievm_dsp_gic_cmd_w[1] : 1'b0;     
assign nfc_nfm_re_no = (tievm_dsp_fpga_pt_mode_en) ? tievm_dsp_fpga_led0_i  : 1'b1;     
assign nfc_nfm_we_no = (tievm_dsp_fpga_pt_mode_en) ? tievm_dsp_gic_stb_w    : 1'b1;
assign nfc_nfm_wp_no = (tievm_dsp_fpga_pt_mode_en) ? tievm_nand_wp_w        : 1'b1;     

assign nfm_nfc_i             = nfm_nfc_io;

assign tievm_dsp_gic_data_io = (tievm_dsp_fpga_pt_mode_en && !ptm_rd_en_nfc) ? nfm_nfc_io :
                               (tievm_gic_top_outdata_en_o )  ?
                                tievm_fpga_dsp_data_w         :  'bz;

assign tievm_fpga_dsp_data_w = tievm_bmc_gic_boot_done_o    ?
                               tievm_gic_dsp_data_o         : tievm_bmc_top_data_w[7:0];

assign tievm_dsp_gic_cmd_io  =  !tievm_bmc_gic_boot_done_o  ? 
                                 tievm_bmc_top_data_w[9:8]  :  'bz;

assign tievm_dsp_gic_stb_io  =  !tievm_bmc_gic_boot_done_o  ? 
                                 tievm_bmc_top_data_w[10]   :  'bz;

assign tievm_gic_dsp_rdy_o   = (!tievm_dsp_fpga_pt_mode_en) ? 
                                 tievm_gic_dsp_ready_w      : nfm_nfc_rb_i;

assign tievm_gic_dsp_ready_w =  tievm_bmc_gic_boot_done_o   ?
                                tievm_gic_dsp_rdy_w         : tievm_bmc_top_data_w[11];

assign {tievm_dsp_fpga_led1_io,tievm_dsp_fpga_led0_io} = !tievm_bmc_gic_boot_done_o   ? 
                                                          tievm_bmc_top_data_w[13:12] :  'bz;

assign tievm_dsp_gic_mode_sel_io  = !tievm_bmc_gic_boot_done_o ? 
                                     tievm_bmc_top_data_w[14]  : 1'bz;

assign tievm_fpga_dsp_uart_intr_o = !tievm_bmc_gic_boot_done_o  ?
                                     tievm_bmc_top_data_w[15]   : tievm_bridge_intrpt_ni;

assign tievm_dsp_gic_data_w  = tievm_dsp_gic_data_io ;
assign tievm_dsp_gic_stb_w   = tievm_dsp_gic_stb_io  ; 
assign tievm_dsp_gic_cmd_w   = tievm_dsp_gic_cmd_io  ;
assign {tievm_dsp_bmc_led1_i,tievm_dsp_bmc_led0_i} = (tievm_bmc_gic_boot_done_o & user_led_en_w) ?
                                                     {tievm_user_led1_w,tievm_user_led0_w      } : 2'b11;

assign tievm_user_sw1_w = tievm_bmc_gic_boot_done_o ? tievm_bmc_top_data_w[7] : 1'b0;
assign tievm_user_sw2_w = tievm_bmc_gic_boot_done_o ? tievm_bmc_top_data_w[8] : 1'b0;

///////////////////////////////////////////////////////////////////////////////
// Logic Implementation for Pass Through Mode.
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_80MHz_clk or negedge sys_rst_sync_clk80_w)
  begin
     if(!sys_rst_sync_clk80_w)
       begin
          tievm_dsp_fpga_pt_mode_en <= 1'b0;
       end
     else if(tievm_bmc_gic_boot_done_o)
       begin
          tievm_dsp_fpga_pt_mode_en <= tievm_dsp_gic_mode_sel_i;
       end
     else
       begin
          tievm_dsp_fpga_pt_mode_en <= 1'b0;
       end
  end

assign tievm_dsp_gic_data_i      = (!tievm_dsp_fpga_pt_mode_en) ?
                                    tievm_dsp_gic_data_w        : 8'h00;

assign tievm_dsp_gic_cmd_i       = (!tievm_dsp_fpga_pt_mode_en) ? 
                                    tievm_dsp_gic_cmd_w         : 2'b00;

assign tievm_dsp_gic_stb_i       = (!tievm_dsp_fpga_pt_mode_en) ? 
                                    tievm_dsp_gic_stb_w         : 1'b0 ;

assign tievm_dsp_fpga_led0_i     = tievm_dsp_fpga_led0_io     ;
assign tievm_dsp_fpga_led1_i     = tievm_dsp_fpga_led1_io     ;

assign tievm_dsp_gic_mode_sel_i  = tievm_dsp_gic_mode_sel_io  ;

// This input is mapped to address 0x22 which is system status register.
assign tievm_user_sw_w = {tievm_board_ver,2'b00,tievm_amc_detect,tievm_user_sw2_w,tievm_user_sw1_w};

///////////////////////////////////////////////////////////////////////////////
//Module instantiation
///////////////////////////////////////////////////////////////////////////////

tievm_gic U_tievm_gic  (
                        .tievm_crm_gic_clka_i            (tievm_80MHz_clk                ),
                        .tievm_crm_gic_clkb_i            (tievm_40MHz_clk                ), 
                        .tievm_crm_gic_rst_ni            (sys_rst_sync_clk80_w           ),

                        .tievm_dsp_gic_data_i            (tievm_dsp_gic_data_i           ),  
                        .tievm_dsp_gic_cmd_i             (tievm_dsp_gic_cmd_i            ),
                        .tievm_dsp_gic_stb_i             (tievm_dsp_gic_stb_i            ),

                        .tievm_nfc_gic_cycle_cnt_l_i     (16'd0                          ),
                        .tievm_nfc_gic_cycle_cnt_h_i     (16'd0                          ),
                        .tievm_nfc_gic_sts_reg_l_i       ( 8'd0                          ),
                        .tievm_nfc_gic_sts_reg_h_i       ( 8'd0                          ),
                        .tievm_nfc_gic_read_id_byte0_i   ( 8'd0                          ),
                        .tievm_nfc_gic_read_id_byte1_i   ( 8'd0                          ),
                        .tievm_nfc_gic_read_id_byte2_i   ( 8'd0                          ),
                        .tievm_nfc_gic_read_id_byte3_i   ( 8'd0                          ), 
                        .tievm_nfc_gic_bad_addr_l_i      ( 8'd0                          ),
                        .tievm_nfc_gic_bad_addr_h_i      ( 8'd0                          ),
                        .tievm_nfc_gic_busy_i            ( 1'b0                          ),
                        .tievm_nfc_gic_rdata_i           ( 8'd0                          ),
                        .tievm_nfc_gic_req_ack_i         ( 1'b0                          ),
                        .tievm_bmc_gic_boot_done_i       (tievm_bmc_gic_boot_done_o      ),
                        .tievm_gic_dsp_data_o            (tievm_gic_dsp_data_o           ),
                        .tievm_gic_top_dsp_rdy_o         (tievm_gic_dsp_rdy_w            ), 
                        .tievm_gic_dsp_nfc_intr_o        (tievm_gic_dsp_nfc_intr_w       ),
                        .tievm_gic_dsp_user_led_en_o     (user_led_en_w                  ),
                        .tievm_gic_dsp_user_led0_o       (tievm_user_led0_w              ),
                        .tievm_gic_dsp_user_led1_o       (tievm_user_led1_w              ),
                        .tievm_gic_top_nand_wp_o         (tievm_nand_wp_w                ),
                        .tievm_gic_nfc_cwr_reg_l_o       (tievm_gic_nfc_cwr_reg_l_o      ),
                        .tievm_gic_nfc_cwr_reg_h_o       (tievm_gic_nfc_cwr_reg_h_o      ),
                        .tievm_gic_nfc_cmd_reg_l_o       (tievm_gic_nfc_cmd_reg_l_o      ),
                        .tievm_gic_nfc_cmd_reg_h_o       (tievm_gic_nfc_cmd_reg_h_o      ),
                        .tievm_gic_nfc_nfm_addr_reg_o    (tievm_gic_nfc_nfm_addr_reg_o   ),
                        .tievm_gic_nfc_on_off_addr_l_o   (tievm_gic_nfc_on_off_addr_l_o  ),
                        .tievm_gic_nfc_on_off_addr_h_o   (tievm_gic_nfc_on_off_addr_h_o  ),
                        .tievm_gic_nfc_rd_timeout_l_o    (tievm_gic_nfc_rd_timeout_l_o   ),
                        .tievm_gic_nfc_rd_timeout_h_o    (tievm_gic_nfc_rd_timeout_h_o   ),
                        .tievm_gic_nfc_wr_timeout_l_o    (tievm_gic_nfc_wr_timeout_l_o   ),
                        .tievm_gic_nfc_wr_timeout_h_o    (tievm_gic_nfc_wr_timeout_h_o   ),
                        .tievm_gic_nfc_erase_timeout_l_o (tievm_gic_nfc_erase_timeout_l_o),
                        .tievm_gic_nfc_erase_timeout_h_o (tievm_gic_nfc_erase_timeout_h_o),
                        .tievm_gic_nfc_cen_no            (tievm_gic_nfc_cen_no           ),
                        .tievm_gic_nfc_rd_req_o          (tievm_gic_nfc_rd_req_o         ),
                        .tievm_gic_nfc_wr_req_o          (tievm_gic_nfc_wr_req_o         ), 
                        .tievm_gic_nfc_wdata_o           (tievm_gic_nfc_wdata_o          ),
                        .tievm_gic_nfc_wdata_valid_o     (tievm_gic_nfc_wdata_valid_o    ),
                        .tievm_gic_top_outdata_en_o      (tievm_gic_top_outdata_en_o     ),
                        .tievm_gic_nfc_sys_addr_o        (nfc_sys_addr_i                 ),
                        .tievm_top_gic_user_sw_i         (tievm_user_sw_w                ),
                        .tievm_gic_nfc_ecc_mode_sel_o    (tievm_gic_nfc_ecc_mode_sel_w   )
                       );

tievm_bmc  U_tievm_bmc (
                        //Clock & Reset Inputs
                        .tievm_fpga_clk_i                (tievm_40MHz_clk                ),
                        .tievm_sys_rst_ni                (cold_rst_sync_clk40_w          ),    
                        .tievm_warm_rst_i                (!warm_rst_sync_clk40_w         ), 

                        // PLL signal
                        .tievm_pll_lock_i                (pll_lock_o                     ),

                        // DIP switches
                        .tievm_dip_sw1_lendian_i         (tievm_dip_sw1_lendian_i        ),
                        .tievm_dip_sw2_btmode0_i         (tievm_dip_sw2_btmode0_i        ),
                        .tievm_dip_sw3_btmode1_i         (tievm_dip_sw3_btmode1_i        ),
                        .tievm_dip_sw4_btmode2_i         (tievm_dip_sw4_btmode2_i        ),
                        .tievm_dip_sw5_btmode3_i         (tievm_dip_sw5_btmode3_i        ),
                        .tievm_dip_sw6_devnum0_i         (tievm_dip_sw6_devnum0_i        ),
                        .tievm_dip_sw7_devnum1_i         (tievm_dip_sw7_devnum1_i        ),
                        .tievm_dip_sw8_usrsw1_i          (tievm_dip_sw8_usrsw1_i         ),
                        .tievm_dip_sw9_usrsw2_i          (tievm_dip_sw9_usrsw2_i         ),
                        .tievm_dip_sw10_cfgp0_i          (tievm_dip_sw10_cfgp0_i         ),
                        .tievm_dip_sw11_cfgp1_i          (tievm_dip_sw11_cfgp1_i         ),
                        .tievm_dip_sw12_cfgp2_i          (tievm_dip_sw12_cfgp2_i         ),
                        .tievm_dip_sw13_cfgp3_i          (tievm_dip_sw13_cfgp3_i         ),
                        .tievm_dip_sw14_cfgp4_i          (tievm_dip_sw14_cfgp4_i         ),
                        .tievm_dip_sw15_hpiwidth_i       (tievm_dip_sw15_hpiwidth_i      ),
                        .tievm_dip_sw16_eclkinsel_i      (tievm_dip_sw16_eclkinsel_i     ),

                        // TI DSP signal inputs
                        .tievm_dsp_fpga_rst_status_i     (tievm_dsp_fpga_rst_status_i    ),

                        // LED inputs
                        .tievm_dsp_bmc_led0_i            (tievm_dsp_bmc_led0_i           ),
                        .tievm_dsp_bmc_led1_i            (tievm_dsp_bmc_led1_i           ),

                        // LED outputs
                        .tievm_user_led1_o               (tievm_user_led1_o              ), 
                        .tievm_user_led2_o               (tievm_user_led2_o              ),

                        // TI DSP signal outputs
                        .tievm_bmc_por_no                (tievm_bmc_por_no               ),
                        .tievm_bmc_reset_no              (tievm_bmc_reset_no             ),

                        //GIC outputs
                        .tievm_bmc_gic_boot_done_o       (tievm_bmc_gic_boot_done_o      ),
                        .tievm_bmc_gic_data_o            (tievm_bmc_top_data_w           )
                       );

///////////////////////////////////////////////////////////////////////////////
// 9-Bit wide counter to implement generation of 10KHz clock for 
// Pseudo Random Sequence Generation.
///////////////////////////////////////////////////////////////////////////////

tievm_clk_counter #(
                   .CNTR_WIDTH     ( 4'd11),
                   .UP_COUNT_LIMIT ('d1999)
                   )
                   clk_cntr_10KHz (
                                   .clk_i      (tievm_40MHz_clk      ),
                                   .arst_ni    (sys_rst_sync_clk40_w ),
                                   .count_out  (                     ),
                                   .clock_o    (tievm_prbs_10KHz_clk ),
                                   .count_up_o (                     )
                                  );

tievm_crm_cm U_tievm_crm_cm (
                             .POWERDOWN (tievm_crm_gic_rst_ni),
                             .CLKA      (tievm_fpga_clk_i    ),
                             .LOCK      (pll_lock_o          ),
                             .GLA       (tievm_40MHz_clk     ),
                             .GLB       (tievm_80MHz_clk     ),
                             .GLC       (tievm_10MHz_clk     )
                            );

assign sys_rst_sync_clk40_w = cold_rst_sync_clk40_w & 
                              warm_rst_sync_clk40_w ;

assign sys_rst_sync_clk80_w = cold_rst_sync_clk80_w & 
                              warm_rst_sync_clk80_w ;

tievm_fpga_reset_sync U_tievm_fpga_cold_reset_sync_clk40( 
                                                         .sys_clk  (tievm_40MHz_clk       ),
                                                         .rst      (tievm_crm_gic_rst_ni  ),
                                                         .rst_sync (cold_rst_sync_clk40_w )
                                                        );

tievm_fpga_reset_sync U_tievm_fpga_warm_reset_sync_clk40( 
                                                         .sys_clk  (tievm_40MHz_clk       ),
                                                         .rst      (tievm_warm_rst_ni     ),
                                                         .rst_sync (warm_rst_sync_clk40_w )
                                                        );


tievm_fpga_reset_sync U_tievm_fpga_cold_reset_sync_clk80( 
                                                         .sys_clk  (tievm_80MHz_clk       ),
                                                         .rst      (tievm_crm_gic_rst_ni  ),
                                                         .rst_sync (cold_rst_sync_clk80_w )
                                                        );

tievm_fpga_reset_sync U_tievm_fpga_warm_reset_sync_clk80( 
                                                         .sys_clk  (tievm_80MHz_clk       ),
                                                         .rst      (tievm_warm_rst_ni     ),
                                                         .rst_sync (warm_rst_sync_clk80_w )
                                                        );

///////////////////////////////////////////////////////////////////////////////
// Logic for Generating 511-Bit Pseudo Random Sequence at 10KHz clock
///////////////////////////////////////////////////////////////////////////////


assign tievm_fpga_prbs_o = (!tievm_fpga_prbs_w) ? 1'b0 : 1'bz; 
assign tievm_fpga_prbs_w = prand_number_gen[8];

always@(posedge tievm_prbs_10KHz_clk or negedge tievm_crm_gic_rst_ni or negedge tievm_warm_rst_ni)
  begin
     if(!tievm_crm_gic_rst_ni || !tievm_warm_rst_ni)
       begin
          prand_number_gen[8:0] <= 9'h1ff;
       end
     else
       begin
          if(prand_number_gen[8:0] == 9'h000)
            begin
               prand_number_gen[8:0] <= 9'h1ff;
            end
          else
            begin
               prand_number_gen[8:1] <= prand_number_gen[7:0];
               prand_number_gen[0]   <= prand_number_gen[0]^prand_number_gen[8];
            end
       end
  end

///////////////////////////////////////////////////////////////////////////////
// AMC detect pin mapped to register bit [2] at 0x22h register of CR
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_80MHz_clk or negedge sys_rst_sync_clk80_w)
  begin
     if(!sys_rst_sync_clk80_w)
       begin
          tievm_amc_detect <= tievm_amc_detect_ni;
          tievm_board_ver  <= tievm_board_ver_i  ;
       end
     else
       begin
          tievm_amc_detect <= tievm_amc_detect   ;
          tievm_board_ver  <= tievm_board_ver    ;
       end
  end

endmodule 

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

