///////////////////////////////////////////////////////////////////////////////                                        
// File name    : tievm_gic_cr.v
// Project      : TI_C6457_LCEVM
// Created On   : 08/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module implements the basic configuration register block
//              : It also has the interrupt generation logic. The necessary ack
//              : generation logic is also included.
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_gic_cr.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.4  2010/09/16 08:05:48  christianfelix_rw
// Revision     : Updated RTL/BITFILE Version to 0x04.
// Revision     :
// Revision     : Revision 1.3  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.2  2010/09/03 10:28:50  christianfelix_rw
// Revision     : RTL/BITFILE Version changed to 0x03.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
///////////////////////////////////////////////////////////////////////////////                                        

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
/////////////////////////////////////////////////////////////////////////////// 

`timescale 1ns/1ps

module tievm_gic_cr(
                    //system inputs
                    tievm_crm_cr_rst_ni         ,
                    tievm_crm_cr_clka_i         ,
                    tievm_crm_cr_clkb_i         ,
                                                       
                    //PMB interface
                    tievm_pmb_cr_reg_sel_i      ,
                    tievm_pmb_cr_wr_en_i        ,
                    tievm_pmb_cr_rd_en_i        ,
                    tievm_pmb_cr_wr_data_i      ,
                    tievm_pmb_cr_tout_err_i     ,
                    tievm_cr_pmb_rd_data_o      ,
                    tievm_cr_pmb_rd_ack_o       ,
                    tievm_cr_pmb_wr_ack_o       ,

                    //GIC module interface
                    tievm_cr_gic_nfc_intr_o     ,
                                                  
                    //NFC interface
                    tievm_cr_nfc_cwr_reg_o      ,
                    tievm_cr_nfc_cmd_reg_o      ,
                    tievm_cr_nfc_nfm_add_reg_o  ,
                    tievm_cr_nfc_on_off_addr_o  ,
                    tievm_cr_nfc_rd_timeout_o   ,
                    tievm_cr_nfc_wr_timeout_o   ,
                    tievm_cr_nfc_erase_timeout_o,
                    tievm_cr_nfc_ecc_mode_sel_o ,
                    tievm_nfc_cr_cycle_cnt_i    ,
                    tievm_nfc_cr_sts_reg_i      ,
                    tievm_nfc_cr_read_id_i      ,    
                    tievm_nfc_cr_bad_addr_i     ,

                    //NIC module interface
                    tievm_cr_nic_nop_cmd_o      ,
                    tievm_cr_nic_pg_wr_cmd_o    ,
                    tievm_cr_nic_pg_rd_cmd_o    ,
                    tievm_cr_nic_sp_wr_cmd_o    ,
                    tievm_cr_nic_sp_rd_cmd_o    ,
                    tievm_cr_nic_bl_er_cmd_o    ,
                    tievm_cr_nic_rd_id_cmd_o    ,
                    tievm_nic_cr_busy_h_to_l_i  ,
                    tievm_cr_nic_sp_rdwr_en_o   ,

                    //top module interface
                    tievm_top_cr_user_sw_i      ,
                    tievm_cr_top_user_led_en_o  ,
                    tievm_cr_top_user_led1_o    ,
                    tievm_cr_top_user_led0_o    , 
                    tievm_cr_top_nand_wp_o
                    );

///////////////////////////////////////////////////////////////////////////////                                        
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////                                        

parameter TIEVM_CR_SEL_WIDTH    = 48;
parameter TIEVM_NAND_ADDR_WIDTH = 32;
parameter TIEVM_CR_WIDTH        =  8;
parameter TIEVM_NFC_ON_OFF_TIME = 16'h0011;
parameter PAGE_WRITE_CMD        =  8'h02;
parameter PAGE_READ_CMD         =  8'h01;
parameter SPARE_AREA_WRITE_CMD  =  8'h82;
parameter SPARE_AREA_READ_CMD   =  8'h81;
parameter BLOCK_ERASE_CMD       =  8'h03;
parameter READ_DEV_ID_CMD       =  8'h06;

///////////////////////////////////////////////////////////////////////////////                                        
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////                                        
input                                    tievm_crm_cr_rst_ni       ;
input                                    tievm_crm_cr_clka_i       ;
input                                    tievm_crm_cr_clkb_i       ;
input [TIEVM_CR_SEL_WIDTH       - 1 : 0] tievm_pmb_cr_reg_sel_i    ;
input                                    tievm_pmb_cr_wr_en_i      ;
input                                    tievm_pmb_cr_rd_en_i      ;
input [(TIEVM_CR_WIDTH * 4)     - 1 : 0] tievm_nfc_cr_cycle_cnt_i  ;
input [(TIEVM_CR_WIDTH * 2)     - 1 : 0] tievm_nfc_cr_sts_reg_i    ;
input [(TIEVM_CR_WIDTH * 4)     - 1 : 0] tievm_nfc_cr_read_id_i    ;
input [(TIEVM_CR_WIDTH * 2)     - 1 : 0] tievm_nfc_cr_bad_addr_i   ;
input  [TIEVM_CR_WIDTH          - 1 : 0] tievm_pmb_cr_wr_data_i    ;
input                                    tievm_nic_cr_busy_h_to_l_i;
input [    TIEVM_CR_WIDTH       - 1 : 0] tievm_top_cr_user_sw_i    ;
input                                    tievm_pmb_cr_tout_err_i   ;

output [TIEVM_CR_WIDTH          - 1 : 0] tievm_cr_pmb_rd_data_o      ; 
output                                   tievm_cr_gic_nfc_intr_o     ; 
output [(TIEVM_CR_WIDTH * 2)    - 1 : 0] tievm_cr_nfc_cwr_reg_o      ; 
output [(TIEVM_CR_WIDTH * 2)    - 1 : 0] tievm_cr_nfc_cmd_reg_o      ; 
output [(TIEVM_CR_WIDTH * 2)    - 1 : 0] tievm_cr_nfc_on_off_addr_o  ; 
output [(TIEVM_CR_WIDTH * 2)    - 1 : 0] tievm_cr_nfc_rd_timeout_o   ; 
output [(TIEVM_CR_WIDTH * 2)    - 1 : 0] tievm_cr_nfc_wr_timeout_o   ; 
output [(TIEVM_CR_WIDTH * 4)    - 1 : 0] tievm_cr_nfc_erase_timeout_o; 
output [TIEVM_NAND_ADDR_WIDTH   - 1 : 0] tievm_cr_nfc_nfm_add_reg_o  ; 
output                                   tievm_cr_nic_nop_cmd_o      ;
output                                   tievm_cr_nic_pg_wr_cmd_o    ;
output                                   tievm_cr_nic_pg_rd_cmd_o    ;
output                                   tievm_cr_nic_sp_wr_cmd_o    ;
output                                   tievm_cr_nic_sp_rd_cmd_o    ;
output                                   tievm_cr_nic_bl_er_cmd_o    ;
output                                   tievm_cr_nic_rd_id_cmd_o    ;
output                                   tievm_cr_pmb_wr_ack_o       ;
output                                   tievm_cr_pmb_rd_ack_o       ;
output                                   tievm_cr_nic_sp_rdwr_en_o   ;
output                                   tievm_cr_nfc_ecc_mode_sel_o ;
output                                   tievm_cr_top_user_led_en_o  ;
output                                   tievm_cr_top_user_led1_o    ;
output                                   tievm_cr_top_user_led0_o    ;
output                                   tievm_cr_top_nand_wp_o      ;

///////////////////////////////////////////////////////////////////////////////                                        
//OUTPUT PORT REGISTERS AND WIRES
///////////////////////////////////////////////////////////////////////////////                                        

reg [TIEVM_CR_WIDTH          - 1 : 0] tievm_cr_pmb_rd_data_o    ; 
reg                                   tievm_cr_pmb_rd_ack_o     ;
reg                                   tievm_cr_pmb_wr_ack_o     ;
reg                                   tievm_cr_nic_nop_cmd_o    ;
reg                                   tievm_cr_nic_pg_wr_cmd_o  ;
reg                                   tievm_cr_nic_pg_rd_cmd_o  ;
reg                                   tievm_cr_nic_sp_wr_cmd_o  ;
reg                                   tievm_cr_nic_sp_rd_cmd_o  ;
reg                                   tievm_cr_nic_bl_er_cmd_o  ;
reg                                   tievm_cr_nic_rd_id_cmd_o  ;
wire                                  tievm_cr_top_user_led_en_o;
wire                                  tievm_cr_top_user_led1_o  ;
wire                                  tievm_cr_top_user_led0_o  ;
wire                                  tievm_cr_top_nand_wp_o    ;

///////////////////////////////////////////////////////////////////////////////                                        
//INTERNAL WIRES AND REGISTERS
///////////////////////////////////////////////////////////////////////////////                                        
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_regll_cb        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reglh_cb        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reghl_cb        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reghh_cb        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cwr_regl_cb          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cwr_regh_cb          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cmd_regl_cb          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cmd_regh_cb          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_rd_timeout_regl_cb    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_rd_timeout_regh_cb    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_wr_timeout_regl_cb    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_wr_timeout_regh_cb    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_regll_cb;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reglh_cb;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reghl_cb;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reghh_cb;
reg [TIEVM_CR_WIDTH      - 1 : 0] tievm_cr_pmb_rd_data_cb   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] tievm_ecc_mode_reg_cb     ;

reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_regll        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reglh        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reghl        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_addr_reghh        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] isr_regl               ;
reg [TIEVM_CR_WIDTH      - 1 : 0] isr_regh               ;
reg [TIEVM_CR_WIDTH      - 1 : 0] bitfile_ver_reg        ;
reg [TIEVM_CR_WIDTH      - 1 : 0] rtl_ver_reg            ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cwr_regl          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cwr_regh          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_sts_regl          ; 
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_sts_regh          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cmd_regl          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nand_cmd_regh          ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_bad_addr_regl      ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_bad_addr_regh      ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_on_off_addr_regl   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_on_off_addr_regh   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_rd_timeout_regl    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_rd_timeout_regh    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_wr_timeout_regl    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_wr_timeout_regh    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_regll;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reglh;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reghl;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_erase_timeout_reghh;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_cycle_cnt_regll    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_cycle_cnt_reglh    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_cycle_cnt_reghl    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_cycle_cnt_reghh    ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_readid_byte0_reg   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_readid_byte1_reg   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_readid_byte2_reg   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] nfc_readid_byte3_reg   ;
reg [TIEVM_CR_WIDTH      - 1 : 0] user_sw_reg            ;
reg                               tievm_nand_wr_protect_cb;
reg                               tievm_user_led_en_cb   ;
reg                               tievm_user_led0_cb     ;
reg                               tievm_user_led1_cb     ;
reg [TIEVM_CR_WIDTH      - 1 : 0] tievm_ecc_mode_reg     ;
reg                               tievm_cr_pmb_wr_ack_cb ;
reg                               tievm_cr_pmb_rd_ack_cb ;
reg                               tievm_nfc_cr_busy_d    ;
reg                               cr_wr_en_d             ;


reg pg_prgm_done_d     ;
reg pg_rd_done_d       ; 
reg cr_rd_d1           ;
reg cr_rd_d2           ;
reg pg_prgm_done_cb    ;
reg pg_rd_done_cb      ;
reg blk_er_done_d      ;
reg blk_er_done_cb     ;

reg devid_rd_done_d    ;
reg devid_rd_done_cb   ;

reg [1:0] rd_ecc_err_d ;
reg [1:0] rd_ecc_err_cb;
wire [1:0] rd_ecc_err_w;

reg mst_tout_err       ;

reg illegal_sp_wr      ;
reg illegal_sp_wr_err  ;

wire devid_rd_done_w   ;

wire blk_er_done_w     ;
wire pg_rd_done_w      ;
wire pg_prgm_done_w    ;

wire ecc_mode_sel_w    ;
wire illegal_sp_wr_w   ;

assign tievm_cr_nfc_cwr_reg_o      = {nand_cwr_regh,nand_cwr_regl}; 
assign tievm_cr_nfc_cmd_reg_o      = {nand_cmd_regh,nand_cmd_regl}; 
assign tievm_cr_nfc_on_off_addr_o  = {nfc_on_off_addr_regh,nfc_on_off_addr_regl}; 
assign tievm_cr_nfc_rd_timeout_o   = {nfc_rd_timeout_regh,nfc_rd_timeout_regl}; 
assign tievm_cr_nfc_wr_timeout_o   = {nfc_wr_timeout_regh,nfc_wr_timeout_regl}; 
assign tievm_cr_nfc_erase_timeout_o= {nfc_erase_timeout_reghh,
                                      nfc_erase_timeout_reghl,
                                      nfc_erase_timeout_reglh,
                                      nfc_erase_timeout_regll
                                      };

assign tievm_cr_nfc_nfm_add_reg_o = {nand_addr_reghh,nand_addr_reghl,
                                     nand_addr_reglh,nand_addr_regll};

assign tievm_cr_nic_sp_rdwr_en_o = nand_cmd_regl[7];


always @(*)
begin

  tievm_cr_nic_nop_cmd_o   = 0;
  tievm_cr_nic_pg_wr_cmd_o = 0;
  tievm_cr_nic_pg_rd_cmd_o = 0;
  tievm_cr_nic_sp_wr_cmd_o = 0;
  tievm_cr_nic_sp_rd_cmd_o = 0;
  tievm_cr_nic_bl_er_cmd_o = 0;
  tievm_cr_nic_rd_id_cmd_o = 0;

  case(nand_cmd_regl)
    PAGE_WRITE_CMD      : tievm_cr_nic_pg_wr_cmd_o = 1'b1;
    PAGE_READ_CMD       : tievm_cr_nic_pg_rd_cmd_o = 1'b1;
    SPARE_AREA_WRITE_CMD: tievm_cr_nic_sp_wr_cmd_o = !ecc_mode_sel_w;
    SPARE_AREA_READ_CMD : tievm_cr_nic_sp_rd_cmd_o = 1'b1;
    BLOCK_ERASE_CMD     : tievm_cr_nic_bl_er_cmd_o = 1'b1;
    READ_DEV_ID_CMD     : tievm_cr_nic_rd_id_cmd_o = 1'b1;
    default             : tievm_cr_nic_nop_cmd_o   = 1'b1;
  endcase

end


///////////////////////////////////////////////////////////////////////////////
//Logic for delaying write pulse by 1 clk cycle
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_crm_cr_clka_i or
         negedge tievm_crm_cr_rst_ni )
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    cr_wr_en_d <= 1'b0;
  end
  else
  begin
    cr_wr_en_d <= tievm_pmb_cr_wr_en_i;
  end
end

///////////////////////////////////////////////////////////////////////////////
//Logic for genrating write/read ack for any register and read enable signal
//for device id register.
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_crm_cr_clka_i or
         negedge tievm_crm_cr_rst_ni)
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    tievm_cr_pmb_wr_ack_o <= 1'b0;
    tievm_cr_pmb_rd_ack_o <= 1'b0;
  end
  else
  begin
    tievm_cr_pmb_wr_ack_o <= tievm_cr_pmb_wr_ack_cb;
    tievm_cr_pmb_rd_ack_o <= tievm_pmb_cr_rd_en_i;
  end
end

always @(*)
begin
  if  ( tievm_cr_nic_pg_wr_cmd_o |
        tievm_cr_nic_sp_wr_cmd_o |
        tievm_cr_nic_pg_rd_cmd_o |
        tievm_cr_nic_sp_rd_cmd_o |
        tievm_cr_nic_bl_er_cmd_o |
        tievm_cr_nic_rd_id_cmd_o
       )
  begin
    tievm_cr_pmb_wr_ack_cb = tievm_nic_cr_busy_h_to_l_i;
  end
  else
  begin
    tievm_cr_pmb_wr_ack_cb = cr_wr_en_d;
  end
end

///////////////////////////////////////////////////////////////////////////////
//Following logic implements the Configuration register write logic
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_crm_cr_clka_i or
         negedge tievm_crm_cr_rst_ni)
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    nand_addr_regll         <= 8'h00 ;
    nand_addr_reglh         <= 8'h00 ;
    nand_addr_reghl         <= 8'h00 ;
    nand_addr_reghh         <= 8'h00 ;
    isr_regl                <= 8'h00 ;
    isr_regh                <= 8'h00 ;
    bitfile_ver_reg         <= 8'h04 ;
    rtl_ver_reg             <= 8'h04 ;
    nand_cwr_regl           <= 8'h00 ;
    nand_cwr_regh           <= 8'h70 ;
    nand_sts_regl           <= 8'h00 ;
    nand_sts_regh           <= 8'h00 ;
    nand_cmd_regl           <= 8'h00 ;
    nand_cmd_regh           <= 8'h00 ;
    nfc_bad_addr_regl       <= 8'h00 ;
    nfc_bad_addr_regh       <= 8'h00 ;
    nfc_rd_timeout_regl     <= 8'hFF ;
    nfc_rd_timeout_regh     <= 8'hFF ;
    nfc_wr_timeout_regl     <= 8'hFF ;
    nfc_wr_timeout_regh     <= 8'hFF ;
    nfc_erase_timeout_regll <= 8'hFF ;
    nfc_erase_timeout_reglh <= 8'hFF ;
    nfc_erase_timeout_reghl <= 8'hFF ;
    nfc_erase_timeout_reghh <= 8'hFF ;
    nfc_cycle_cnt_regll     <= 8'h00 ;
    nfc_cycle_cnt_reglh     <= 8'h00 ;
    nfc_cycle_cnt_reghl     <= 8'h00 ;
    nfc_cycle_cnt_reghh     <= 8'h00 ;
    nfc_readid_byte0_reg    <= 8'h00 ;
    nfc_readid_byte1_reg    <= 8'h00 ;
    nfc_readid_byte2_reg    <= 8'h00 ;
    nfc_readid_byte3_reg    <= 8'h00 ;
   {nfc_on_off_addr_regh,nfc_on_off_addr_regl}       <= TIEVM_NFC_ON_OFF_TIME ;
    user_sw_reg             <= 8'h00 ;
    tievm_ecc_mode_reg      <= 8'b0000_0001;
  end
  else
  begin
    nand_addr_regll         <= nand_addr_regll_cb;
    nand_addr_reglh         <= nand_addr_reglh_cb;
    nand_addr_reghl         <= nand_addr_reghl_cb;
    nand_addr_reghh         <= nand_addr_reghh_cb;
    isr_regl                <= 8'd0;
    isr_regh                <= 8'd0;
    bitfile_ver_reg         <= bitfile_ver_reg;
    rtl_ver_reg             <= rtl_ver_reg;
    nand_cwr_regl           <= nand_cwr_regl_cb;
    nand_cwr_regh           <= nand_cwr_regh_cb;
    nand_sts_regl[2:0]      <= {blk_er_done_cb,pg_prgm_done_cb,pg_rd_done_cb} ;
    nand_sts_regl[7:3]      <= tievm_nfc_cr_sts_reg_i[7:3];
    nand_sts_regh[1:0]      <= {mst_tout_err,devid_rd_done_cb};
    nand_sts_regh[3:2]      <= rd_ecc_err_cb;
    nand_sts_regh[4]        <= tievm_nfc_cr_sts_reg_i[12];
    nand_sts_regh[5]        <= illegal_sp_wr_err;
    nand_sts_regh[7:6]      <= tievm_nfc_cr_sts_reg_i[15:14];
    nand_cmd_regl           <= nand_cmd_regl_cb;
    nand_cmd_regh           <= nand_cmd_regh_cb;
    nfc_bad_addr_regl       <= tievm_nfc_cr_bad_addr_i[7:0];
    nfc_bad_addr_regh       <= tievm_nfc_cr_bad_addr_i[15:8];
    nfc_on_off_addr_regl    <= nfc_on_off_addr_regl;
    nfc_on_off_addr_regh    <= nfc_on_off_addr_regh;
    nfc_rd_timeout_regl     <= nfc_rd_timeout_regl_cb;
    nfc_rd_timeout_regh     <= nfc_rd_timeout_regh_cb;
    nfc_wr_timeout_regl     <= nfc_wr_timeout_regl_cb;
    nfc_wr_timeout_regh     <= nfc_wr_timeout_regh_cb;
    nfc_erase_timeout_regll <= nfc_erase_timeout_regll_cb;
    nfc_erase_timeout_reglh <= nfc_erase_timeout_reglh_cb;
    nfc_erase_timeout_reghl <= nfc_erase_timeout_reghl_cb;
    nfc_erase_timeout_reghh <= nfc_erase_timeout_reghh_cb;
    nfc_cycle_cnt_regll     <= tievm_nfc_cr_cycle_cnt_i[7:0];
    nfc_cycle_cnt_reglh     <= tievm_nfc_cr_cycle_cnt_i[15:8];
    nfc_cycle_cnt_reghl     <= tievm_nfc_cr_cycle_cnt_i[23:16];
    nfc_cycle_cnt_reghh     <= tievm_nfc_cr_cycle_cnt_i[31:24];
    nfc_readid_byte0_reg    <= tievm_nfc_cr_read_id_i[7:0];
    nfc_readid_byte1_reg    <= tievm_nfc_cr_read_id_i[15:8];
    nfc_readid_byte2_reg    <= tievm_nfc_cr_read_id_i[23:16];
    nfc_readid_byte3_reg    <= tievm_nfc_cr_read_id_i[31:24];

    user_sw_reg             <= tievm_top_cr_user_sw_i;
    tievm_ecc_mode_reg      <= tievm_ecc_mode_reg_cb;
  end
end

assign ecc_mode_sel_w = tievm_ecc_mode_reg[0];

assign tievm_cr_nfc_ecc_mode_sel_o = ecc_mode_sel_w;
///////////////////////////////////////////////////////////////////////////////
//Logic for detecting low to high edge for page programme done and page read
//done
///////////////////////////////////////////////////////////////////////////////



always @(posedge tievm_crm_cr_clkb_i or
         negedge tievm_crm_cr_rst_ni)
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    pg_prgm_done_d <= 1'b0;
    pg_rd_done_d   <= 1'b0;
    blk_er_done_d  <= 1'b0;
    devid_rd_done_d <= 1'b0;
    rd_ecc_err_d    <= 'b0;

    cr_rd_d1    <= 1'b0;
    cr_rd_d2    <= 1'b0;
  end
  else
  begin
    pg_rd_done_d <= tievm_nfc_cr_sts_reg_i[0];
    pg_prgm_done_d <= tievm_nfc_cr_sts_reg_i[1];
    blk_er_done_d  <= tievm_nfc_cr_sts_reg_i[2];
    devid_rd_done_d <= tievm_nfc_cr_sts_reg_i[8];
    rd_ecc_err_d <= tievm_nfc_cr_sts_reg_i[11:10];

    cr_rd_d1 <= tievm_pmb_cr_rd_en_i ; 
    cr_rd_d2 <= cr_rd_d1 ; 
  end
end  

assign pg_rd_done_w = !pg_rd_done_d & tievm_nfc_cr_sts_reg_i[0];
assign pg_prgm_done_w = !pg_prgm_done_d & tievm_nfc_cr_sts_reg_i[1];
assign blk_er_done_w  = !blk_er_done_d & tievm_nfc_cr_sts_reg_i[2];
assign devid_rd_done_w  = !devid_rd_done_d & tievm_nfc_cr_sts_reg_i[8];

assign rd_ecc_err_w[0] = !rd_ecc_err_d[0] & tievm_nfc_cr_sts_reg_i[10];
assign rd_ecc_err_w[1] = !rd_ecc_err_d[1] & tievm_nfc_cr_sts_reg_i[11];
 
always @(*)
begin
  if(pg_rd_done_w)
  begin
    pg_rd_done_cb = 1'b1;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[10])
  begin
    pg_rd_done_cb = 1'b0;
  end
  else
  begin
    pg_rd_done_cb = nand_sts_regl[0];
  end
end

always @(*)
begin
  if(pg_prgm_done_w)
  begin
    pg_prgm_done_cb = pg_prgm_done_w;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[10])
  begin
    pg_prgm_done_cb = 1'b0;
  end
  else
  begin
    pg_prgm_done_cb = nand_sts_regl[1];
  end
end

always @(*)
begin
  if(blk_er_done_w)
  begin
    blk_er_done_cb = blk_er_done_w;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[10])
  begin
    blk_er_done_cb = 1'b0;
  end
  else
  begin
    blk_er_done_cb  = nand_sts_regl[2];
  end
end

always @(*)
begin
  if(devid_rd_done_w)
  begin
    devid_rd_done_cb = 1'b1;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[11])
  begin
   devid_rd_done_cb = 1'b0;
  end
  else
  begin
   devid_rd_done_cb  = nand_sts_regh[0];
  end
end

always @(*)
begin
  if(tievm_pmb_cr_tout_err_i)
  begin
    mst_tout_err = 1'b1;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[11])
  begin
   mst_tout_err = 1'b0;
  end
  else
  begin
   mst_tout_err  = nand_sts_regh[1];
  end
end

always @(*)
begin
  if(rd_ecc_err_w[0])
  begin
    rd_ecc_err_cb[0] = 1'b1 ;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[11])
  begin
    rd_ecc_err_cb[0] = 1'b0;
  end
  else
  begin
    rd_ecc_err_cb[0] = nand_sts_regh[2];
  end
end

always @(*)
begin
  if(rd_ecc_err_w[1])
  begin
    rd_ecc_err_cb[1] = 1'b1;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[11])
  begin
    rd_ecc_err_cb[1] = 1'b0;
  end
  else
  begin
    rd_ecc_err_cb[1] = nand_sts_regh[3];
  end
end

always @(*)
begin
  if(illegal_sp_wr)
  begin
    illegal_sp_wr_err = 1'b1;
  end
  else if(cr_rd_d2 & tievm_pmb_cr_reg_sel_i[11])
  begin
    illegal_sp_wr_err = 1'b0;
  end
  else
  begin
    illegal_sp_wr_err = nand_sts_regh[5];
  end
end

///////////////////////////////////////////////////////////////////////////////

assign illegal_sp_wr_w = ( (nand_cmd_regl == 8'h82) && ecc_mode_sel_w );

always @(posedge tievm_crm_cr_clka_i or
         negedge tievm_crm_cr_rst_ni)
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    illegal_sp_wr <= 'b0;
  end
  else
  begin
    illegal_sp_wr <= illegal_sp_wr_w;
  end
end  

always @(*)
begin
  if(tievm_pmb_cr_reg_sel_i[0] & tievm_pmb_cr_wr_en_i)
  begin
    nand_addr_regll_cb = tievm_pmb_cr_wr_data_i; 
  end
  else
  begin
    nand_addr_regll_cb = nand_addr_regll; 
  end

  if(tievm_pmb_cr_reg_sel_i[1] & tievm_pmb_cr_wr_en_i)
  begin
    nand_addr_reglh_cb = tievm_pmb_cr_wr_data_i; 
  end
  else
  begin
    nand_addr_reglh_cb = nand_addr_reglh; 
  end

  if(tievm_pmb_cr_reg_sel_i[2] & tievm_pmb_cr_wr_en_i)
  begin
    nand_addr_reghl_cb = tievm_pmb_cr_wr_data_i; 
  end
  else
  begin
    nand_addr_reghl_cb = nand_addr_reghl; 
  end

  if(tievm_pmb_cr_reg_sel_i[3] & tievm_pmb_cr_wr_en_i)
  begin
    nand_addr_reghh_cb = tievm_pmb_cr_wr_data_i; 
  end
  else
  begin
    nand_addr_reghh_cb = nand_addr_reghh; 
  end

  if(tievm_pmb_cr_reg_sel_i[8] & tievm_pmb_cr_wr_en_i)
  begin
    nand_cwr_regl_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nand_cwr_regl_cb = nand_cwr_regl;
  end

  if(tievm_pmb_cr_reg_sel_i[9] & tievm_pmb_cr_wr_en_i)
  begin
    nand_cwr_regh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nand_cwr_regh_cb = nand_cwr_regh;
  end

  if(tievm_nic_cr_busy_h_to_l_i | illegal_sp_wr)
  begin
    nand_cmd_regl_cb[6:0] = 7'h00;
    nand_cmd_regl_cb[7]   = nand_cmd_regl[7];
  end
  else if(tievm_pmb_cr_reg_sel_i[12] & tievm_pmb_cr_wr_en_i)
  begin
    nand_cmd_regl_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nand_cmd_regl_cb = nand_cmd_regl;
  end

  if(tievm_nic_cr_busy_h_to_l_i)
  begin
    nand_cmd_regh_cb[7:0] = 8'h00;
  end  
  else if(tievm_pmb_cr_reg_sel_i[13] & tievm_pmb_cr_wr_en_i)
  begin
    nand_cmd_regh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nand_cmd_regh_cb = nand_cmd_regh;
  end

  if(tievm_pmb_cr_reg_sel_i[18] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_rd_timeout_regl_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_rd_timeout_regl_cb = nfc_rd_timeout_regl;
  end

  if(tievm_pmb_cr_reg_sel_i[19] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_rd_timeout_regh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_rd_timeout_regh_cb = nfc_rd_timeout_regh;
  end

  if(tievm_pmb_cr_reg_sel_i[20] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_wr_timeout_regl_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_wr_timeout_regl_cb = nfc_wr_timeout_regl;
  end

  if(tievm_pmb_cr_reg_sel_i[21] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_wr_timeout_regh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_wr_timeout_regh_cb = nfc_wr_timeout_regh;
  end

  if(tievm_pmb_cr_reg_sel_i[22] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_erase_timeout_regll_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_erase_timeout_regll_cb = nfc_erase_timeout_regll;
  end

  if(tievm_pmb_cr_reg_sel_i[23] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_erase_timeout_reglh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_erase_timeout_reglh_cb = nfc_erase_timeout_reglh;
  end

  if(tievm_pmb_cr_reg_sel_i[24] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_erase_timeout_reghl_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_erase_timeout_reghl_cb = nfc_erase_timeout_reghl;
  end

  if(tievm_pmb_cr_reg_sel_i[25] & tievm_pmb_cr_wr_en_i)
  begin
    nfc_erase_timeout_reghh_cb = tievm_pmb_cr_wr_data_i;
  end
  else
  begin
    nfc_erase_timeout_reghh_cb = nfc_erase_timeout_reghh;
  end

  if(tievm_pmb_cr_reg_sel_i[35] & tievm_pmb_cr_wr_en_i)
  begin
    tievm_ecc_mode_reg_cb = tievm_pmb_cr_wr_data_i; 
  end
  else
  begin
    tievm_ecc_mode_reg_cb = tievm_ecc_mode_reg; 
  end

end

assign tievm_cr_top_user_led_en_o  = tievm_user_led_en_cb    ;
assign tievm_cr_top_user_led1_o    = tievm_user_led1_cb      ;
assign tievm_cr_top_user_led0_o    = tievm_user_led0_cb      ;
assign tievm_cr_top_nand_wp_o      = tievm_nand_wr_protect_cb;

always @(posedge tievm_crm_cr_clka_i or negedge tievm_crm_cr_rst_ni)
  begin
     if(!tievm_crm_cr_rst_ni)
       begin
          tievm_user_led_en_cb     <= 1'b0;
          tievm_user_led1_cb       <= 1'b1;
          tievm_user_led0_cb       <= 1'b1;
          tievm_nand_wr_protect_cb <= 1'b1;
       end
     else
       begin
          tievm_user_led_en_cb     <= nand_cwr_regh[7];
          tievm_user_led1_cb       <= nand_cwr_regh[6];
          tievm_user_led0_cb       <= nand_cwr_regh[5];
          tievm_nand_wr_protect_cb <= nand_cwr_regh[4];
      end
  end

///////////////////////////////////////////////////////////////////////////////
//Following logic implements the Configuration register read logic
//////////////////////////////////////////////////////////////////////////////

always @(*)
begin

  tievm_cr_pmb_rd_data_cb = tievm_cr_pmb_rd_data_o ;

  case(tievm_pmb_cr_reg_sel_i)
  
  48'h0000_0000_0001 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_addr_regll;         
  48'h0000_0000_0002 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_addr_reglh;         
  48'h0000_0000_0004 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_addr_reghl;         
  48'h0000_0000_0008 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_addr_reghh;         
  48'h0000_0000_0010 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = isr_regl;               
  48'h0000_0000_0020 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = isr_regh;               
  48'h0000_0000_0040 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = bitfile_ver_reg;        
  48'h0000_0000_0080 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = rtl_ver_reg;            
  48'h0000_0000_0100 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_cwr_regl;          
  48'h0000_0000_0200 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_cwr_regh;          
  48'h0000_0000_0400 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_sts_regl;          
  48'h0000_0000_0800 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_sts_regh;          
  48'h0000_0000_1000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_cmd_regl;          
  48'h0000_0000_2000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nand_cmd_regh;          
  48'h0000_0000_4000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_bad_addr_regl;      
  48'h0000_0000_8000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_bad_addr_regh;      
  48'h0000_0001_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_on_off_addr_regl;   
  48'h0000_0002_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_on_off_addr_regh;   
  48'h0000_0004_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_rd_timeout_regl;    
  48'h0000_0008_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_rd_timeout_regh;    
  48'h0000_0010_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_wr_timeout_regl;    
  48'h0000_0020_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_wr_timeout_regh;    
  48'h0000_0040_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_erase_timeout_regll;
  48'h0000_0080_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_erase_timeout_reglh;
  48'h0000_0100_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_erase_timeout_reghl;
  48'h0000_0200_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_erase_timeout_reghh;
  48'h0000_0400_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_cycle_cnt_regll;    
  48'h0000_0800_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_cycle_cnt_reglh;    
  48'h0000_1000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_cycle_cnt_reghl;    
  48'h0000_2000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_cycle_cnt_reghh;    
  48'h0000_4000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_readid_byte0_reg;   
  48'h0000_8000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_readid_byte1_reg;   
  48'h0001_0000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_readid_byte2_reg;   
  48'h0002_0000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = nfc_readid_byte3_reg;   
  48'h0004_0000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = user_sw_reg;   
  48'h0008_0000_0000 : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = tievm_ecc_mode_reg;   
  default            : if(tievm_pmb_cr_rd_en_i) tievm_cr_pmb_rd_data_cb = 8'h00;

  endcase
end

always @(posedge tievm_crm_cr_clka_i or
         negedge tievm_crm_cr_rst_ni)
begin
  if(!tievm_crm_cr_rst_ni)
  begin
    tievm_cr_pmb_rd_data_o <= 8'd0;
  end
  else
  begin
    tievm_cr_pmb_rd_data_o <= tievm_cr_pmb_rd_data_cb;
  end
end

/////////////////////////////////////////////////////////////////////////////
//Interrupt controller
/////////////////////////////////////////////////////////////////////////////
//wire [31:0] nfc_intr_w;

//assign nfc_intr_w = {nand_sts_regh,nand_sts_regl} & {nand_cwr_regh,nand_cwr_regl};

assign tievm_cr_pmb_nfc_intr_o = 1'b0;

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

