///////////////////////////////////////////////////////////////////////////////
// File name    : tievm_gic_nic.v
// Project      : TI_C6457_LCEVM
// Created On   : 06/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module implements the control logic for interfacing with
//              : the Nand flash controller it contains following modules
//              : 1. tievm_nic_nicfsm
//              : 2. tievm_nic_nicfifo
//              :
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_gic_nic.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module tievm_gic_nic (
 
                      //system input
                      tievm_crm_nic_clka_i,
                      tievm_crm_nic_clkb_i,
                      tievm_crm_nic_rst_ni,
                                                   
                      //PMB module interface
                      tievm_pmb_nic_wr_en_i,
                      tievm_pmb_nic_rd_en_i,
                      tievm_nic_pmb_wr_ack_o,
                      tievm_nic_pmb_rd_ack_o,
                      tievm_pmb_nic_wr_data_i,
                      tievm_nic_pmb_rd_data_o,
                      tievm_pmb_nic_fifo_rd_i,
                      
                      //NFC Interface
                      tievm_nic_nfc_cen_no,
                      tievm_nic_nfc_rd_req_o,
                      tievm_nic_nfc_wr_req_o,
                      tievm_nic_nfc_wr_data_o,
                      tievm_nic_nfc_wdata_valid_o,
                      tievm_nfc_nic_busy_i,
                      tievm_nfc_nic_rd_data_i,
                      tievm_nfc_nic_req_ack_i,

                      tievm_nic_nfc_sys_addr_o,
                                                   
                      //CRB Interface
                      tievm_cr_nic_nop_cmd_i,
                      tievm_cr_nic_pg_wr_cmd_i,
                      tievm_cr_nic_pg_rd_cmd_i,
                      tievm_cr_nic_sp_wr_cmd_i,
                      tievm_cr_nic_sp_rd_cmd_i,
                      tievm_cr_nic_bl_er_cmd_i,
                      tievm_cr_nic_rd_id_cmd_i,
                      tievm_nic_cr_busy_h_to_l_o,
                      tievm_cr_nic_sp_rdwr_en_i
                       );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////

parameter TIEVM_GPIO_BUS_WIDTH = 8;

///////////////////////////////////////////////////////////////////////////////
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////
input                                  tievm_crm_nic_clka_i        ;  
input                                  tievm_crm_nic_clkb_i        ; 
input                                  tievm_crm_nic_rst_ni        ; 
input                                  tievm_pmb_nic_wr_en_i       ; 
input                                  tievm_pmb_nic_rd_en_i       ; 
input                                  tievm_nfc_nic_busy_i        ; 
input [TIEVM_GPIO_BUS_WIDTH  - 1 : 0]  tievm_nfc_nic_rd_data_i     ; 
input                                  tievm_nfc_nic_req_ack_i     ; 
input                                  tievm_cr_nic_nop_cmd_i      ; 
input                                  tievm_cr_nic_pg_wr_cmd_i    ; 
input                                  tievm_cr_nic_pg_rd_cmd_i    ; 
input                                  tievm_cr_nic_sp_wr_cmd_i    ; 
input                                  tievm_cr_nic_sp_rd_cmd_i    ; 
input                                  tievm_cr_nic_bl_er_cmd_i    ; 
input                                  tievm_cr_nic_rd_id_cmd_i    ; 
input [TIEVM_GPIO_BUS_WIDTH  - 1 : 0]  tievm_pmb_nic_wr_data_i     ; 
input                                  tievm_pmb_nic_fifo_rd_i     ;
input                                  tievm_cr_nic_sp_rdwr_en_i   ;
                                                                       
output                                 tievm_nic_pmb_wr_ack_o     ;
output                                 tievm_nic_pmb_rd_ack_o     ;
output                                 tievm_nic_nfc_cen_no       ;
output                                 tievm_nic_nfc_rd_req_o     ;
output                                 tievm_nic_nfc_wr_req_o     ;
output                                 tievm_nic_nfc_wdata_valid_o;
output [TIEVM_GPIO_BUS_WIDTH - 1 : 0]  tievm_nic_pmb_rd_data_o    ;
output [TIEVM_GPIO_BUS_WIDTH - 1 : 0]  tievm_nic_nfc_wr_data_o    ;
output                                 tievm_nic_cr_busy_h_to_l_o ;
output [31 : 0]  tievm_nic_nfc_sys_addr_o    ;



///////////////////////////////////////////////////////////////////////////////
//OUTPUT PORT REGISTERS
///////////////////////////////////////////////////////////////////////////////

reg                                tievm_nic_pmb_wr_ack_o     ;
reg                                tievm_nic_pmb_rd_ack_o     ;
reg [TIEVM_GPIO_BUS_WIDTH - 1 : 0] tievm_nic_nfc_wr_data_o    ;
reg                                tievm_nic_nfc_rd_req_o     ;

///////////////////////////////////////////////////////////////////////////////
//INTERNAL WIRES AND REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg tievm_nic_pmb_wr_ack_cb;
reg tievm_nic_pmb_rd_ack_cb;
reg tievm_nfc_nic_busy_d;
reg                                    page_prg_done;    
reg                                    page_rd_done;
reg                                    busy_high_to_low;
reg                                    busy_low_to_high;

reg nic_porta_wren_d;
reg nic_porta_rden_d1;
reg nic_porta_rden_d2;

reg tievm_nic_nicfsm_wr_req;
reg tievm_nic_nicfsm_rd_req;

reg tievm_cr_nic_pg_rd_cmd_d;
reg tievm_cr_nic_pg_wr_cmd_d;

reg [7:0] tievm_nic_nfc_wr_data_d ;

reg tievm_nic_nfc_wr_req_d;

wire tievm_nic_nfc_sp_wr_req_w ;
reg tievm_nic_nfc_sp_wr_req_d0 ; 
reg tievm_nic_nfc_sp_wr_req_d1 ;
reg tievm_nic_nfc_sp_wr_req_d2 ;
reg tievm_nic_nfc_sp_wr_req_d3 ;


wire spare_cmd_w;



wire nic_porta_wren_w;
wire nic_porta_rden_w;
wire tievm_nicfsm_nicfifo_wr_en_w ;
wire tievm_nicfsm_nicfifo_rd_en_w ;
wire [11:0] tievm_nic_nicfifo_fullthrh_w ;
wire [11:0] tievm_nic_nicfifo_emptythrh_w;
wire tievm_nicfifo_nic_fifofull_w ;
wire tievm_nicfifo_nic_fifoempty_w;
wire new_pg_rd_req_w ;
wire new_pg_wr_req_w ;

wire [7:0] tievm_nic_nfc_wr_data_w;
wire       tievm_nic_nfc_wr_req_w;
wire       tievm_nic_nfc_rd_req_w;

wire req_ack_w;
 
wire [31:0] tievm_nic_nfc_sys_addr_o /*synthesis syn_keep = 1*/; 

wire [1:0] tievm_nic_nfc_sys_addr_w /*synthesis syn_keep = 1*/;


assign tievm_nic_cr_busy_h_to_l_o = busy_high_to_low;

assign tievm_nic_nfc_sys_addr_o = {21'b0,tievm_nic_nfc_sys_addr_w,9'b0};



tievm_nic_nicfifo tievm_nic_nicfifo_U(
          .tievm_crm_nicfifo_clka_i     (tievm_crm_nic_clka_i     ),
          .tievm_crm_nicfifo_clkb_i     (tievm_crm_nic_clkb_i     ),                             
          .tievm_crm_nicfifo_rst_ni     (tievm_crm_nic_rst_ni     ),                             
          .tievm_pmb_nicfifo_wr_en_i    (nic_porta_wren_w),                                                   
          .tievm_pmb_nicfifo_rd_en_i    (nic_porta_rden_w),                                                  
          .tievm_pmb_nicfifo_wr_data_i  (tievm_pmb_nic_wr_data_i  ),                             
          .tievm_nicfifo_pmb_rd_data_o  (tievm_nic_pmb_rd_data_o  ),                             
          .tievm_nfc_nicfifo_wr_data_i  (tievm_nfc_nic_rd_data_i  ),                             
          .tievm_nicfifo_nfc_rd_data_o  (tievm_nic_nfc_wr_data_w  ),                         
          .tievm_nic_nicfifo_porta_wr_ni(tievm_nic_nicfifo_porta_wr_w),                         
          .tievm_nicfsm_nicfifo_wr_en_i (tievm_nicfsm_nicfifo_wr_en_w ),                                                 
          .tievm_nicfsm_nicfifo_rd_en_i (tievm_nicfsm_nicfifo_rd_en_w ),                                                          
          .tievm_nic_nicfifo_fullthrh_i (tievm_nic_nicfifo_fullthrh_w ),                         
          .tievm_nic_nicfifo_emptythrh_i(tievm_nic_nicfifo_emptythrh_w),                           
          .tievm_nicfifo_nic_fifofull_o (tievm_nicfifo_nic_fifofull_w ),                          
          .tievm_nicfifo_nic_fifoempty_o(tievm_nicfifo_nic_fifoempty_w),

          .tievm_nic_nicfifo_sync_rst_i (busy_high_to_low)

          );

tievm_nic_nicfsm tievm_nic_nicfsm_U (
     .tievm_crm_nicfsm_clk_i            (tievm_crm_nic_clkb_i            ),
     .tievm_crm_nicfsm_rst_ni           (tievm_crm_nic_rst_ni           ),                                 
     .tievm_nic_nicfsm_wr_req_i         (tievm_nic_nicfsm_wr_req         ),                                
     .tievm_nic_nicfsm_rd_req_i         (tievm_nic_nicfsm_rd_req         ),                                
     .tievm_nic_nicfsm_task_done_i      (busy_high_to_low                ),                                
     .tievm_nicfifo_nicfsm_fifo_full_i  (tievm_nicfifo_nic_fifofull_w  ),                                  
     .tievm_nicfifo_nicfsm_fifo_empty_i (tievm_nicfifo_nic_fifoempty_w ),                                  
     .tievm_nicfsm_nicfifo_wr_en_o      (tievm_nicfsm_nicfifo_wr_en_w      ),                              
     .tievm_nicfsm_nicfifo_rd_en_o      (tievm_nicfsm_nicfifo_rd_en_w      ),                              
     .tievm_nicfsm_nfc_cen_no           (tievm_nic_nfc_cen_no           ),                                 
     .tievm_nicfsm_nfc_rd_req_o         (tievm_nic_nfc_rd_req_w         ),                                 
     .tievm_nicfsm_nfc_wr_req_o         (tievm_nic_nfc_wr_req_w         ),                                 
     .tievm_nicfsm_nfc_wdata_valid_o    (tievm_nic_nfc_wdata_valid_o    ),                                 
     .tievm_nfc_nicfsm_req_ack_i        (req_ack_w),//tievm_nfc_nic_req_ack_i        ),
     .tievm_nicfsm_nfc_sys_addr_o       (tievm_nic_nfc_sys_addr_w )
                                    );

///////////////////////////////////////////////////////////////////////////////
//Logic for providing direction to the FIFO
///////////////////////////////////////////////////////////////////////////////
assign tievm_nic_nicfifo_porta_wr_w = (tievm_cr_nic_pg_rd_cmd_i | 
                                       tievm_cr_nic_sp_rd_cmd_i |
                                       tievm_pmb_nic_fifo_rd_i 
                                       );

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//Logic for giving appropriate threshold for different cmds
///////////////////////////////////////////////////////////////////////////////


assign spare_cmd_w = tievm_cr_nic_sp_wr_cmd_i | tievm_cr_nic_sp_rd_cmd_i ;

assign req_ack_w = tievm_cr_nic_sp_rd_cmd_i ? !tievm_nfc_nic_req_ack_i  :
                                               tievm_nfc_nic_req_ack_i   ;


assign tievm_nic_nicfifo_fullthrh_w  = tievm_cr_nic_sp_rdwr_en_i ?    12'd64:
                                                                     12'd2048 ;
assign tievm_nic_nicfifo_emptythrh_w = tievm_nic_nicfifo_fullthrh_w ; 
///////////////////////////////////////////////////////////////////////////////

//assign tievm_nic_nfc_wr_data_o = spare_cmd_w ? tievm_nic_nfc_wr_data_w :
//                                              tievm_nic_nfc_wr_data_d ;

///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////

always @(negedge tievm_crm_nic_clkb_i or
         negedge tievm_crm_nic_rst_ni
         )
begin
  if(!tievm_crm_nic_rst_ni)
  begin
   tievm_nic_nfc_wr_data_o <= 8'd0;
   tievm_nic_nfc_wr_req_d  <= 1'b0;
   tievm_nic_nfc_rd_req_o  <= 1'b0;
  end
  else
  begin
   tievm_nic_nfc_wr_data_o <= tievm_nic_nfc_wr_data_w;
   tievm_nic_nfc_wr_req_d  <= tievm_nic_nfc_wr_req_w;
   tievm_nic_nfc_rd_req_o  <= tievm_nic_nfc_rd_req_w;
  end
end  



///////////////////////////////////////////////////////////////////////////////

assign tievm_nic_nfc_wr_req_o = spare_cmd_w ? tievm_nic_nfc_sp_wr_req_w :
                                             tievm_nic_nfc_wr_req_d ;


assign tievm_nic_nfc_sp_wr_req_w = tievm_nic_nfc_sp_wr_req_d1 |
                                   tievm_nic_nfc_wr_req_w     ;

always @(posedge tievm_crm_nic_clkb_i or
         negedge tievm_crm_nic_rst_ni
         )
begin
  if(!tievm_crm_nic_rst_ni)
  begin
    tievm_nic_nfc_sp_wr_req_d0 <= 1'b0;
    tievm_nic_nfc_sp_wr_req_d1 <= 1'b0;
    tievm_nic_nfc_sp_wr_req_d2 <= 1'b0;
    tievm_nic_nfc_sp_wr_req_d3 <= 1'b0;
  end
  else
  begin
   tievm_nic_nfc_sp_wr_req_d0 <= tievm_nic_nfc_wr_req_d;
   tievm_nic_nfc_sp_wr_req_d1 <= tievm_nic_nfc_sp_wr_req_d0;
   tievm_nic_nfc_sp_wr_req_d2 <= tievm_nic_nfc_sp_wr_req_d1;
   tievm_nic_nfc_sp_wr_req_d3 <= tievm_nic_nfc_sp_wr_req_d2;
  end
end

///////////////////////////////////////////////////////////////////////////////
//Logic for providing read / write ack
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_crm_nic_clka_i or
         negedge tievm_crm_nic_rst_ni
         )
begin
  if(!tievm_crm_nic_rst_ni)
  begin
    tievm_nic_pmb_wr_ack_o <= 1'b0;
    tievm_nic_pmb_rd_ack_o <= 1'b0;
  end
  else
  begin
    tievm_nic_pmb_wr_ack_o <= tievm_pmb_nic_wr_en_i;
    tievm_nic_pmb_rd_ack_o <= tievm_pmb_nic_rd_en_i;
  end
end


// This logic detects the high -> low transition of busy signal
//busy from NFC goes from high -> low when the page is written 
//in the NFM and status register is updated.

always @(posedge tievm_crm_nic_clkb_i or //signal is in the clkb domain (i.e
                                         //40Mhz)
         negedge tievm_crm_nic_rst_ni 
         )
begin 
if(!tievm_crm_nic_rst_ni)
  begin
    tievm_nfc_nic_busy_d <= 1'b0;
  end
  else
  begin
    tievm_nfc_nic_busy_d <= tievm_nfc_nic_busy_i ;
  end
end


always @(tievm_nfc_nic_busy_i or
         tievm_nfc_nic_busy_d
         )
begin
  busy_high_to_low = tievm_nfc_nic_busy_d & (!tievm_nfc_nic_busy_i) ;
  busy_low_to_high = !tievm_nfc_nic_busy_d & tievm_nfc_nic_busy_i ;
end



///////////////////////////////////////////////////////////////////////////////
//Logic for generating write signal for the FIFO 
///////////////////////////////////////////////////////////////////////////////
//for porta i.e page write



always @(posedge tievm_crm_nic_clka_i or
         negedge tievm_crm_nic_rst_ni 
         )
begin
  if(!tievm_crm_nic_rst_ni)
  begin
    nic_porta_wren_d <= 1'b0;
  end   
  else
  begin
    nic_porta_wren_d <= tievm_pmb_nic_wr_en_i;
  end
end

assign nic_porta_wren_w = tievm_pmb_nic_wr_en_i & !(nic_porta_wren_d);
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//Logic for generating read signal for the FIFO 
///////////////////////////////////////////////////////////////////////////////
//for porta

always @(posedge tievm_crm_nic_clka_i or
         negedge tievm_crm_nic_rst_ni 
         )
begin
  if(!tievm_crm_nic_rst_ni)
  begin
    nic_porta_rden_d1 <= 1'b0;
  end   
  else
  begin
    nic_porta_rden_d1 <= tievm_pmb_nic_rd_en_i;
  end
end


assign nic_porta_rden_w = tievm_pmb_nic_rd_en_i & (!nic_porta_rden_d1) ;
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// Logic for generating wr/rd request for NIC FSM
///////////////////////////////////////////////////////////////////////////////

always @(posedge tievm_crm_nic_clkb_i or //signal is in the clkb domain (i.e
                                         //40Mhz)
         negedge tievm_crm_nic_rst_ni 
         )
begin 
if(!tievm_crm_nic_rst_ni)
  begin
    tievm_nic_nicfsm_wr_req <= 1'b0;
    tievm_nic_nicfsm_rd_req <= 1'b0;
  end
  else
  begin
    tievm_nic_nicfsm_wr_req <= tievm_cr_nic_pg_wr_cmd_i | 
                               tievm_cr_nic_sp_wr_cmd_i     ;
    tievm_nic_nicfsm_rd_req <= tievm_cr_nic_pg_rd_cmd_i |
                               tievm_cr_nic_sp_rd_cmd_i      ;
  end
end

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

