///////////////////////////////////////////////////////////////////////////////
// File name    : tievm_gic_pmb.v
// Project      : TI_C6457_LCEVM
// Created On   : 08/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module contains the basic FSM for GPIO I/F
//              : It also has the decoder for the Configuration
//              : register block. Also implements the command, address and input
//              : data register.
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_gic_pmb.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

module tievm_gic_pmb(
                     tievm_crm_pmb_clk_i ,
                     tievm_crm_pmb_rst_ni,
                     
                     //DSP-GPIO to FPGA Interface  
                     tievm_dsp_pmb_data_i,
                     tievm_dsp_pmb_cmd_i ,
                     tievm_dsp_pmb_stb_i ,
                     tievm_pmb_dsp_data_o,  
                     tievm_pmb_dsp_rdy_o ,
                                                    
                     //NIC Interface
                     tievm_pmb_nic_wr_en_o,
                     tievm_pmb_nic_rd_en_o,
                     tievm_pmb_nic_wr_data_o,
                     tievm_nic_pmb_rd_data_i, 
                     tievm_nic_pmb_rd_ack_i ,
                     tievm_nic_pmb_wr_ack_i ,
		     tievm_pmb_nic_fifo_rd_o,
                     
                                                    
                     //CR Block interface
                     tievm_pmb_cr_reg_sel_o,
                     tievm_pmb_cr_wr_en_o,
                     tievm_pmb_cr_rd_en_o,
                     tievm_pmb_cr_wr_data_o,
                     tievm_cr_pmb_rd_data_i,
                     tievm_cr_pmb_wr_ack_i,
                     tievm_cr_pmb_rd_ack_i,
		     tievm_pmb_cr_tout_err_o,
                                                    
                     //BMC Interface
                     tievm_bmc_pmb_boot_done_i,

                     //TOP module interface
                     tievm_pmb_top_outdata_en_o                            
                    );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////

parameter TIEVM_GPIO_DATA_WIDTH = 8;
parameter TIEVM_GPIO_CMD_WIDTH  = 2;
parameter TIEVM_CR_WIDTH        = 8;
parameter TIEVM_BMP_WIDTH       = 16;
parameter TIEVM_CR_SEL_WIDTH    = 48;

parameter NAND_ADDR_REG_BYTE0  =8'h00;
parameter NAND_ADDR_REG_BYTE1  =8'h01;
parameter NAND_ADDR_REG_BYTE2  =8'h02;
parameter NAND_ADDR_REG_BYTE4  =8'h03;
parameter ISR_REGL             =8'h04;                                         
parameter ISR_REGH             =8'h05;
parameter BITFILE_VER_REG      =8'h06;
parameter RTL_VER_REG          =8'h07;
parameter NAND_CWR_REGL        =8'h08; 
parameter NAND_CWR_REGH        =8'h09; 
parameter NAND_STS_REGL        =8'h0A; 
parameter NAND_STS_REGH        =8'h0B;                                         
parameter NAND_CMD_REGL        =8'h0C;                                         
parameter NAND_CMD_REGH        =8'h0D; 
parameter NFC_BAD_ADDR_REGL    =8'h0E;
parameter NFC_BAD_ADDR_REGH    =8'h0F;
parameter NFC_ON_OFF_ADDRL     =8'h10; 
parameter NFC_ON_OFF_ADDRH     =8'h11;
parameter NFC_RD_TIMEOUTLL      =8'h12; 
parameter NFC_RD_TIMEOUTLH      =8'h13;
parameter NFC_WR_TIMEOUTHL      =8'h14; 
parameter NFC_WR_TIMEOUTHH      =8'h15; 
parameter NFC_ERASE_TIMEOUTLL  =8'h16; 
parameter NFC_ERASE_TIMEOUTLH  =8'h17; 
parameter NFC_ERASE_TIMEOUTHL  =8'h18; 
parameter NFC_ERASE_TIMEOUTHH  =8'h19; 
parameter NFC_CYCLE_CNT_REGLL  =8'h1A; 
parameter NFC_CYCLE_CNT_REGLH  =8'h1B; 
parameter NFC_CYCLE_CNT_REGHL  =8'h1C; 
parameter NFC_CYCLE_CNT_REGHH  =8'h1D; 
parameter NFC_READID_BYTE0_REG =8'h1E; 
parameter NFC_READID_BYTE1_REG =8'h1F; 
parameter NFC_READID_BYTE2_REG =8'h20; 
parameter NFC_READID_BYTE3_REG =8'h21; 
parameter USER_SW_REG          =8'h22; 
parameter TIEVM_ECC_MODE_REG   =8'h23; 

///////////////////////////////////////////////////////////////////////////////
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////

input                                  tievm_crm_pmb_clk_i       ;
input                                  tievm_crm_pmb_rst_ni      ;
input  [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_dsp_pmb_data_i      ;
input  [TIEVM_GPIO_CMD_WIDTH  - 1 : 0] tievm_dsp_pmb_cmd_i       ;
input                                  tievm_dsp_pmb_stb_i       ;
input  [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_nic_pmb_rd_data_i   ;
input                                  tievm_nic_pmb_rd_ack_i    ;
input                                  tievm_nic_pmb_wr_ack_i    ;
input                                  tievm_cr_pmb_wr_ack_i     ;
input                                  tievm_cr_pmb_rd_ack_i     ;
input  [TIEVM_CR_WIDTH        - 1 : 0] tievm_cr_pmb_rd_data_i    ;
input                                  tievm_bmc_pmb_boot_done_i ; 
 
output [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_pmb_nic_wr_data_o   ;
output [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_pmb_dsp_data_o      ;
output                                 tievm_pmb_dsp_rdy_o       ; 
output                                 tievm_pmb_nic_wr_en_o     ; 
output                                 tievm_pmb_nic_rd_en_o     ; 
output [TIEVM_CR_SEL_WIDTH    - 1 : 0] tievm_pmb_cr_reg_sel_o    ;
output                                 tievm_pmb_cr_wr_en_o      ; 
output                                 tievm_pmb_cr_rd_en_o      ; 
output [TIEVM_CR_WIDTH        - 1 : 0] tievm_pmb_cr_wr_data_o    ; 
output                                 tievm_pmb_top_outdata_en_o;
output                                 tievm_pmb_nic_fifo_rd_o   ;
output                                 tievm_pmb_cr_tout_err_o   ;

///////////////////////////////////////////////////////////////////////////////
//OUTPUT PORT REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg    [TIEVM_CR_SEL_WIDTH - 1 : 0]     tievm_pmb_cr_reg_sel_o   ;
reg                                 tievm_pmb_pmbfsm_fifo_sel;
reg [TIEVM_GPIO_DATA_WIDTH - 1 : 0] pmb_add_reg;
reg [TIEVM_GPIO_DATA_WIDTH - 1 : 0] pmb_data_reg;
reg [TIEVM_GPIO_CMD_WIDTH  - 1 : 0] pmb_cmd_reg;

///////////////////////////////////////////////////////////////////////////////
//INTERNAL WIRES AND REGISTERS
///////////////////////////////////////////////////////////////////////////////
wire                                   pmb_add_reg_en_w ;
wire                                   pmb_data_reg_en_w ;
wire                                   pmb_cmd_reg_en_w ;
wire                                   tievm_pmbfsm_pmb_data_sel_w;
wire   [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_pmb_rd_data_w        ;



assign tievm_pmb_dsp_data_o = tievm_pmb_rd_data_w         ;
                              

assign tievm_pmb_rd_data_w  = tievm_pmb_pmbfsm_fifo_sel ?
                              tievm_nic_pmb_rd_data_i     : 
                              tievm_cr_pmb_rd_data_i      ;


assign tievm_pmb_nic_fifo_rd_o = (tievm_pmb_pmbfsm_fifo_sel &
                                  (pmb_cmd_reg == 2'b10)) ;

tievm_pmb_pmbfsm tievm_pmb_pmbfsm_U (
            .tievm_crm_pmbfsm_clk_i          (tievm_crm_pmb_clk_i),
            .tievm_crm_pmbfsm_rst_ni         (tievm_crm_pmb_rst_ni),                         
            .tievm_dsp_pmbfsm_cmd_i          (pmb_cmd_reg),                          		     
            .tievm_dsp_pmbfsm_stb_i          (tievm_dsp_pmb_stb_i),                         
            .tievm_pmbfsm_dsp_data_sel_o     (tievm_pmbfsm_pmb_data_sel_w),                         
            .tievm_pmbfsm_dsp_rdy_o          (tievm_pmb_dsp_rdy_o),                         
            .tievm_pmbfsm_nic_wr_en_o        (tievm_pmb_nic_wr_en_o),                         
            .tievm_pmbfsm_nic_rd_en_o        (tievm_pmb_nic_rd_en_o),                         
            .tievm_nic_pmbfsm_rd_ack_i       (tievm_nic_pmb_rd_ack_i),                         
            .tievm_nic_pmbfsm_wr_ack_i       (tievm_nic_pmb_wr_ack_i),                         
            .tievm_cr_pmbfsm_wr_ack_i        (tievm_cr_pmb_wr_ack_i),                         
            .tievm_cr_pmbfsm_rd_ack_i        (tievm_cr_pmb_rd_ack_i),                         
            .tievm_pmbfsm_cr_wr_en_o         (tievm_pmb_cr_wr_en_o),                         
            .tievm_pmbfsm_cr_rd_en_o         (tievm_pmb_cr_rd_en_o),                         
            .tievm_bmc_pmbfsm_boot_done_i    (tievm_bmc_pmb_boot_done_i),                         
            .tievm_pmbfsm_top_cmdreg_en_o    (pmb_cmd_reg_en_w),                         
            .tievm_pmbfsm_top_addreg_en_o    (pmb_add_reg_en_w),                         
            .tievm_pmbfsm_top_indatareg_en_o (pmb_data_reg_en_w),                         
            .tievm_pmbfsm_top_outdatareg_en_o(tievm_pmb_top_outdata_en_o),                         
	    .tievm_pmb_pmbfsm_fifo_sel_i     (tievm_pmb_pmbfsm_fifo_sel),
	    .tievm_pmbfsm_cr_tout_err_o      (tievm_pmb_cr_tout_err_o)
                                     );



//////////////////////////////////////////////////////////////////////////////
//I/O FLOPs for storing the input data into respective registers.
//////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_pmb_clk_i or
         negedge tievm_crm_pmb_rst_ni
	 )
begin
  if(!tievm_crm_pmb_rst_ni)
  begin
    pmb_add_reg <= 8'd0;
  end
  else
  begin
    if(pmb_add_reg_en_w)
      pmb_add_reg <= tievm_dsp_pmb_data_i ;
  end
end

always @(posedge tievm_crm_pmb_clk_i or
         negedge tievm_crm_pmb_rst_ni
	 )
begin
  if(!tievm_crm_pmb_rst_ni)
  begin
    pmb_data_reg <= 8'd0;
  end
  else
  begin
    if(pmb_data_reg_en_w)
      pmb_data_reg <= tievm_dsp_pmb_data_i ;
  end
end

assign tievm_pmb_cr_wr_data_o = pmb_data_reg  ;
assign tievm_pmb_nic_wr_data_o = pmb_data_reg ;

always @(posedge tievm_crm_pmb_clk_i or
         negedge tievm_crm_pmb_rst_ni
	 )
begin
  if(!tievm_crm_pmb_rst_ni)
  begin
    pmb_cmd_reg <= 2'd0;
  end
  else
  begin
    if(pmb_cmd_reg_en_w)
      pmb_cmd_reg <= tievm_dsp_pmb_cmd_i ;
  end
end



//////////////////////////////////////////////////////////////////////////////
//Address decoder for Configuration memory
//////////////////////////////////////////////////////////////////////////////
always @(pmb_add_reg)
begin
  if(pmb_add_reg[7])
  begin
  tievm_pmb_pmbfsm_fifo_sel = 1;
  tievm_pmb_cr_reg_sel_o = 32'd0;
  end
  else
  begin
  tievm_pmb_pmbfsm_fifo_sel = 0;

    case(pmb_add_reg[7:0])
      NAND_ADDR_REG_BYTE0  : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0001;
      NAND_ADDR_REG_BYTE1  : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0002;
      NAND_ADDR_REG_BYTE2  : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0004;
      NAND_ADDR_REG_BYTE4  : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0008;
      ISR_REGL             : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0010;                                         
      ISR_REGH             : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0020;
      BITFILE_VER_REG      : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0040;
      RTL_VER_REG          : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0080;
      NAND_CWR_REGL        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0100; 
      NAND_CWR_REGH        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0200; 
      NAND_STS_REGL        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0400; 
      NAND_STS_REGH        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_0800;                                         
      NAND_CMD_REGL        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_1000;                                         
      NAND_CMD_REGH        : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_2000; 
      NFC_BAD_ADDR_REGL    : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_4000; 
      NFC_BAD_ADDR_REGH    : tievm_pmb_cr_reg_sel_o = 48'h0000_0000_8000;
      NFC_ON_OFF_ADDRL     : tievm_pmb_cr_reg_sel_o = 48'h0000_0001_0000; 
      NFC_ON_OFF_ADDRH     : tievm_pmb_cr_reg_sel_o = 48'h0000_0002_0000;
      NFC_RD_TIMEOUTLL     : tievm_pmb_cr_reg_sel_o = 48'h0000_0004_0000; 
      NFC_RD_TIMEOUTLH     : tievm_pmb_cr_reg_sel_o = 48'h0000_0008_0000; 
      NFC_WR_TIMEOUTHL     : tievm_pmb_cr_reg_sel_o = 48'h0000_0010_0000; 
      NFC_WR_TIMEOUTHH     : tievm_pmb_cr_reg_sel_o = 48'h0000_0020_0000; 
      NFC_ERASE_TIMEOUTLL  : tievm_pmb_cr_reg_sel_o = 48'h0000_0040_0000; 
      NFC_ERASE_TIMEOUTLH  : tievm_pmb_cr_reg_sel_o = 48'h0000_0080_0000; 
      NFC_ERASE_TIMEOUTHL  : tievm_pmb_cr_reg_sel_o = 48'h0000_0100_0000; 
      NFC_ERASE_TIMEOUTHH  : tievm_pmb_cr_reg_sel_o = 48'h0000_0200_0000; 
      NFC_CYCLE_CNT_REGLL  : tievm_pmb_cr_reg_sel_o = 48'h0000_0400_0000; 
      NFC_CYCLE_CNT_REGLH  : tievm_pmb_cr_reg_sel_o = 48'h0000_0800_0000; 
      NFC_CYCLE_CNT_REGHL  : tievm_pmb_cr_reg_sel_o = 48'h0000_1000_0000; 
      NFC_CYCLE_CNT_REGHH  : tievm_pmb_cr_reg_sel_o = 48'h0000_2000_0000; 
      NFC_READID_BYTE0_REG : tievm_pmb_cr_reg_sel_o = 48'h0000_4000_0000; 
      NFC_READID_BYTE1_REG : tievm_pmb_cr_reg_sel_o = 48'h0000_8000_0000; 
      NFC_READID_BYTE2_REG : tievm_pmb_cr_reg_sel_o = 48'h0001_0000_0000; 
      NFC_READID_BYTE3_REG : tievm_pmb_cr_reg_sel_o = 48'h0002_0000_0000;
      USER_SW_REG          : tievm_pmb_cr_reg_sel_o = 48'h0004_0000_0000;
      TIEVM_ECC_MODE_REG   : tievm_pmb_cr_reg_sel_o = 48'h0008_0000_0000;
      default              : tievm_pmb_cr_reg_sel_o = 48'h8000_0000_0000;
    endcase

  end

end

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

