///////////////////////////////////////////////////////////////////////////////
// File name    : tievm_nic_nicfifo.v
// Project      : TI_C6457_LCEVM
// Created On   : 09/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module implements the bidirectional FIFO of 2048/64 bytes
//              : along with necessary control logic
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_nic_nicfifo.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 100 ps

module tievm_nic_nicfifo(
                         tievm_crm_nicfifo_clka_i,
                         tievm_crm_nicfifo_clkb_i,
                         tievm_crm_nicfifo_rst_ni,
                         
                         //PMB Interface
                         tievm_pmb_nicfifo_wr_en_i,
                         tievm_pmb_nicfifo_rd_en_i,
                         tievm_pmb_nicfifo_wr_data_i,
                         tievm_nicfifo_pmb_rd_data_o,
                                                                         
                                                                         
                         //NFC Interface
                         tievm_nfc_nicfifo_wr_data_i,
                         tievm_nicfifo_nfc_rd_data_o,
                                                                         
                                                                                               
                         //NIC interface
                         tievm_nic_nicfifo_porta_wr_ni,
			 tievm_nicfsm_nicfifo_wr_en_i,
                         tievm_nicfsm_nicfifo_rd_en_i,
			 tievm_nic_nicfifo_fullthrh_i,
                         tievm_nic_nicfifo_emptythrh_i,
                         tievm_nicfifo_nic_fifofull_o,
                         tievm_nicfifo_nic_fifoempty_o,
			 tievm_nic_nicfifo_sync_rst_i,
			 tievm_nicfifo_nic_porta_wrcount_o
                        );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////

parameter TIEVM_GPIO_DATA_WIDTH = 8;

///////////////////////////////////////////////////////////////////////////////
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////
input                                  tievm_crm_nicfifo_clka_i;  
input                                  tievm_crm_nicfifo_clkb_i;  
input                                  tievm_crm_nicfifo_rst_ni;  
input                                  tievm_pmb_nicfifo_wr_en_i; 
input                                  tievm_pmb_nicfifo_rd_en_i; 
input  [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_pmb_nicfifo_wr_data_i;
input                                  tievm_nicfsm_nicfifo_wr_en_i;
input                                  tievm_nicfsm_nicfifo_rd_en_i;
input  [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_nfc_nicfifo_wr_data_i;
input  [11:0]                          tievm_nic_nicfifo_fullthrh_i;
input  [11:0]                          tievm_nic_nicfifo_emptythrh_i;
input                                  tievm_nic_nicfifo_porta_wr_ni;
input                                  tievm_nic_nicfifo_sync_rst_i;
                                                                      
output [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_nicfifo_nfc_rd_data_o;
output                                 tievm_nicfifo_nic_fifofull_o;
output                                 tievm_nicfifo_nic_fifoempty_o;
output [TIEVM_GPIO_DATA_WIDTH - 1 : 0] tievm_nicfifo_pmb_rd_data_o;
output [11:0]                          tievm_nicfifo_nic_porta_wrcount_o;

///////////////////////////////////////////////////////////////////////////////
//OUTPUT PORT REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg                                    tievm_nicfifo_nic_fifofull_o;  
reg                                    tievm_nicfifo_nic_fifoempty_o; 

///////////////////////////////////////////////////////////////////////////////
//INTERNAL WIRES AND REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg    [10:0]                          porta_addr;
reg    [10:0]                          portb_addr;
reg    [10:0]                          porta_addr_d;
reg    [10:0]                          portb_addr_d;
reg    [11:0]                          porta_wrcount; 
reg    [11:0]                          porta_rdcount; 
reg    [11:0]                          portb_wrcount; 
reg    [11:0]                          portb_rdcount;
reg    [11:0]                          porta_wrcount_d; 
reg    [11:0]                          porta_rdcount_d; 
reg    [11:0]                          portb_wrcount_d; 
reg    [11:0]                          portb_rdcount_d;

wire                                   porta_rdwr_en_w;
wire                                   portb_rdwr_en_w;

assign porta_rdwr_en_w = tievm_pmb_nicfifo_wr_en_i |
                         tievm_pmb_nicfifo_rd_en_i   ;

assign portb_rdwr_en_w = tievm_nicfsm_nicfifo_wr_en_i |
                         tievm_nicfsm_nicfifo_rd_en_i   ;

assign tievm_nicfifo_nic_porta_wrcount_o = porta_wrcount;		 

///////////////////////////////////////////////////////////////////////////////
//2K Dual port RAM instantiation
///////////////////////////////////////////////////////////////////////////////
tievm_nic_dpram tievm_nic_dpram_U(
                  .DINA  (tievm_pmb_nicfifo_wr_data_i),
                  .DOUTA (tievm_nicfifo_pmb_rd_data_o),
                  .DINB  (tievm_nfc_nicfifo_wr_data_i),
                  .DOUTB (tievm_nicfifo_nfc_rd_data_o),
                  .ADDRA (porta_addr),
                  .ADDRB (portb_addr),
                  .RWA   (tievm_nic_nicfifo_porta_wr_ni),
                  .RWB   (!tievm_nic_nicfifo_porta_wr_ni),
                  .BLKA  (porta_rdwr_en_w),
                  .BLKB  (portb_rdwr_en_w),
                  .CLKA  (tievm_crm_nicfifo_clka_i),
                  .CLKB  (tievm_crm_nicfifo_clkb_i),
                  .RESET (tievm_crm_nicfifo_rst_ni) 
                  );
///////////////////////////////////////////////////////////////////////////////
//Logic on the port a side for read/write
///////////////////////////////////////////////////////////////////////////////
always @(*)
begin
  if(tievm_nic_nicfifo_sync_rst_i)
  begin
    porta_addr_d = 11'd0;
    porta_wrcount_d = 12'd0;
    porta_rdcount_d = 12'd0;
  end
  else if(porta_rdcount == tievm_nic_nicfifo_emptythrh_i)
  begin
    porta_addr_d = 11'd0;
    porta_wrcount_d = 12'd0;
    porta_rdcount_d = 12'd0;
  end
  else if(tievm_pmb_nicfifo_wr_en_i)
  begin
    porta_addr_d = porta_addr + 1;
    porta_wrcount_d = porta_wrcount + 1;
    porta_rdcount_d = porta_rdcount;
  end
  else if (tievm_pmb_nicfifo_rd_en_i)
  begin
    porta_addr_d = porta_addr + 1;
    porta_wrcount_d = porta_wrcount;
    porta_rdcount_d = porta_rdcount + 1;
  end
  else
  begin
    porta_addr_d = porta_addr;
    porta_wrcount_d = porta_wrcount ;
    porta_rdcount_d = porta_rdcount;
  end
end         

always @(posedge tievm_crm_nicfifo_clka_i or 
         negedge tievm_crm_nicfifo_rst_ni   )
begin
  if(!tievm_crm_nicfifo_rst_ni)
  begin
    porta_addr <= 11'd0;
    porta_wrcount <= 12'd0;
    porta_rdcount <= 12'd0;
  end
  else
  begin
    porta_addr    <= porta_addr_d;
    porta_wrcount <= porta_wrcount_d;
    porta_rdcount <= porta_rdcount_d;
  end
end



///////////////////////////////////////////////////////////////////////////////
//Logic on the port b side for read/write
///////////////////////////////////////////////////////////////////////////////
always @(*)
begin
  if(tievm_nic_nicfifo_sync_rst_i)
  begin
    portb_addr_d = 11'd0;
    portb_wrcount_d = 12'd0;
    portb_rdcount_d = 12'd0;
  end
  else if(tievm_nicfsm_nicfifo_wr_en_i)
  begin
    portb_addr_d    = portb_addr + 1;
    portb_wrcount_d = portb_wrcount + 1;
    portb_rdcount_d = portb_rdcount;
  end
  else if (tievm_nicfsm_nicfifo_rd_en_i)
  begin
    portb_addr_d    = portb_addr + 1;
    portb_wrcount_d = portb_wrcount ;
    portb_rdcount_d = portb_rdcount + 1;
  end
  else
  begin
    portb_addr_d = portb_addr;
    portb_wrcount_d = portb_wrcount;
    portb_rdcount_d = portb_rdcount;
  end
end         

always @(posedge tievm_crm_nicfifo_clkb_i or 
         negedge tievm_crm_nicfifo_rst_ni   )
begin
  if(!tievm_crm_nicfifo_rst_ni)
  begin
    portb_addr <= 11'd0;
    portb_wrcount <= 12'd0;
    portb_rdcount <= 12'd0;
  end
  else
  begin
    portb_addr <= portb_addr_d;
    portb_wrcount <= portb_wrcount_d;
    portb_rdcount <= portb_rdcount_d;
  end
end



///////////////////////////////////////////////////////////////////////////////
//Logic for FIFO full/emtpy generation
///////////////////////////////////////////////////////////////////////////////

always @(*)
begin
  if( !tievm_nic_nicfifo_porta_wr_ni) 
  begin

   if(porta_wrcount == tievm_nic_nicfifo_fullthrh_i)
   begin
    tievm_nicfifo_nic_fifofull_o = 1;// sync to clka
   end
   else
   begin
    tievm_nicfifo_nic_fifofull_o = 0; 
   end

   if(portb_rdcount == tievm_nic_nicfifo_emptythrh_i)
   begin
    tievm_nicfifo_nic_fifoempty_o = 1;// sync to clkb
   end
   else
   begin
    tievm_nicfifo_nic_fifoempty_o = 0;
   end

  end//if( !tievm_nic_nicfifo_porta_wr_ni)
  else
  begin

    if(portb_wrcount == tievm_nic_nicfifo_fullthrh_i)
    begin
      tievm_nicfifo_nic_fifofull_o = 1;//sync to clkb
    end
    else
    begin
      tievm_nicfifo_nic_fifofull_o = 0;
    end

    if(porta_rdcount == tievm_nic_nicfifo_emptythrh_i)
    begin
      tievm_nicfifo_nic_fifoempty_o = 1;//sync to clka
    end
    else
    begin
      tievm_nicfifo_nic_fifoempty_o = 0;
    end

  end//else if(tievm_nic_nicfifo_porta_wr_ni)

end

endmodule 

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

