///////////////////////////////////////////////////////////////////////////////
// File name    : tievm_nic_nicfsm.v
// Project      : TI_C6457_LCEVM
// Created On   : 07/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module implements the basic FSM for interfacing with
//              : with the CIF interface of NFC IP
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_nic_nicfsm.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
// 
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module tievm_nic_nicfsm (
                                    //system inputs
                                    tievm_crm_nicfsm_clk_i,
                                    tievm_crm_nicfsm_rst_ni,
                                                                    
                                    //NIC module interface
                                    tievm_nic_nicfsm_wr_req_i,
                                    tievm_nic_nicfsm_rd_req_i,
                                    tievm_nic_nicfsm_task_done_i,

                                    //NIC FIFO module interface
                                    tievm_nicfifo_nicfsm_fifo_full_i,
                                    tievm_nicfifo_nicfsm_fifo_empty_i,
                                    tievm_nicfsm_nicfifo_wr_en_o,
                                    tievm_nicfsm_nicfifo_rd_en_o,
                                                                    
                                    //NFC interface
                                    tievm_nicfsm_nfc_cen_no,
                                    tievm_nicfsm_nfc_rd_req_o,
                                    tievm_nicfsm_nfc_wr_req_o,
                                    tievm_nicfsm_nfc_wdata_valid_o,
                                    tievm_nicfsm_nfc_sys_addr_o,
                                    tievm_nfc_nicfsm_req_ack_i
                                    );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////

parameter IDLE_STATE = 3'b000;
parameter WRITE_STATE = 3'b001;
parameter WR_DELAY_STATE = 3'b010; 
parameter READ_STATE = 3'b011;
parameter RD_DELAY_STATE = 3'b100; 
parameter NFC_BUSY = 3'b101;
parameter DELAY_STATE = 3'b110;
parameter ERROR_STATE = 3'b111;

///////////////////////////////////////////////////////////////////////////////
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////
input tievm_crm_nicfsm_clk_i       ;
input tievm_crm_nicfsm_rst_ni      ;
input tievm_nic_nicfsm_wr_req_i    ;
input tievm_nic_nicfsm_rd_req_i    ;
input tievm_nicfifo_nicfsm_fifo_full_i ;
input tievm_nicfifo_nicfsm_fifo_empty_i;
input tievm_nfc_nicfsm_req_ack_i       ;
input tievm_nic_nicfsm_task_done_i  ;

                                
output tievm_nicfsm_nfc_cen_no       ;
output tievm_nicfsm_nfc_rd_req_o     ;
output tievm_nicfsm_nfc_wr_req_o     ;
output tievm_nicfsm_nfc_wdata_valid_o;
output tievm_nicfsm_nicfifo_wr_en_o;
output tievm_nicfsm_nicfifo_rd_en_o;
output [1:0] tievm_nicfsm_nfc_sys_addr_o;

///////////////////////////////////////////////////////////////////////////////
//INTERNAL WIRES AND REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg tievm_nicfsm_nfc_cen_no       ;
reg tievm_nicfsm_nfc_rd_req_o     ;
reg tievm_nicfsm_nfc_wr_req_o     ;
//reg tievm_nicfsm_nfc_wdata_valid_o;
//reg tievm_nicfsm_nicfifo_wr_en_o;
//reg tievm_nicfsm_nicfifo_rd_en_o;
reg [1:0] tievm_nicfsm_nfc_sys_addr_o /*synthesis syn_keep = 1*/;

reg [2:0] nicfsm_state;
reg [2:0] nicfsm_nxt_state;

reg nfc_ce;
reg nfc_rd_req;
reg nfc_wr_req;
reg nfc_wdata_valid;
reg nicfifo_wr_en;
reg nicfifo_rd_en;

reg delay_cnt_rst ;
wire delay_cnt_up_w;

reg [9:0] burst_cnt;
reg [9:0] burst_cnt_d;

reg [1:0]nfc_sys_addr /*synthesis syn_keep = 1*/;
//reg tievm_nfc_nicfsm_req_ack_i; 

wire burst_cnt_up ;
reg burst_cnt_up_d ;


assign burst_cnt_up = (burst_cnt == 10'd511) ;


///////////////////////////////////////////////////////////////////////////////
//4 bit wide counter to implement delays between read/write of 512 bytes on the
//CIF interface
///////////////////////////////////////////////////////////////////////////////

tievm_cntr #(.CNTR_WIDTH(3'd4),
             .UP_COUNT(10)
             )
intra_burst_latency_cntr
       (
        .clk_i     (tievm_crm_nicfsm_clk_i) ,                    
        .rst_n_i   (tievm_crm_nicfsm_rst_ni) ,                    
        .sc_i      (delay_cnt_rst) ,                          
        .count_o   () ,                        
        .count_up_o(delay_cnt_up_w)
        );



///////////////////////////////////////////////////////////////////////////////
//logic for generating one clock delayed req ack for page read.
///////////////////////////////////////////////////////////////////////////////
//always @(posedge tievm_crm_nicfsm_clk_i or
//         negedge tievm_crm_nicfsm_rst_ni
//       )
//begin
//  if(!tievm_crm_nicfsm_rst_ni)
//  begin
//    tievm_nfc_nicfsm_req_ack_i <= 1'b0;
//  end
//  else
//  begin
//    tievm_nfc_nicfsm_req_ack_i <= tievm_nfc_nicfsm_req_ack_i;
//  end
//end

///////////////////////////////////////////////////////////////////////////////
//logic for generating one clock delayed count up
///////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_nicfsm_clk_i or
         negedge tievm_crm_nicfsm_rst_ni
         )
begin
  if(!tievm_crm_nicfsm_rst_ni)
  begin
    burst_cnt_up_d <= 1'b0;
  end
  else
  begin
    burst_cnt_up_d <= burst_cnt_up;
  end
end


///////////////////////////////////////////////////////////////////////////////
//Following logic implements the basic FSM
///////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_nicfsm_clk_i or
         negedge tievm_crm_nicfsm_rst_ni
         )
begin
  if(!tievm_crm_nicfsm_rst_ni)
  begin
   tievm_nicfsm_nfc_cen_no       <= 1'b1;
   tievm_nicfsm_nfc_rd_req_o     <= 1'b0;
   tievm_nicfsm_nfc_wr_req_o     <= 1'b0;
 //  tievm_nicfsm_nfc_wdata_valid_o<= 1'b0;
 //  tievm_nicfsm_nicfifo_wr_en_o  <= 1'b0;
  // tievm_nicfsm_nicfifo_rd_en_o  <= 1'b0;
   burst_cnt                     <= 10'd0;
   tievm_nicfsm_nfc_sys_addr_o   <= 2'd0;
   nicfsm_state                  <= 3'b00;

  end
  else
  begin
    tievm_nicfsm_nfc_cen_no       <= nfc_ce;
    tievm_nicfsm_nfc_rd_req_o     <= nfc_rd_req;
    tievm_nicfsm_nfc_wr_req_o     <= nfc_wr_req;
  //  tievm_nicfsm_nfc_wdata_valid_o <=  nfc_wdata_valid;
 //   tievm_nicfsm_nicfifo_wr_en_o   <=  nicfifo_wr_en;
  //  tievm_nicfsm_nicfifo_rd_en_o   <=  nicfifo_rd_en;
    burst_cnt                     <= burst_cnt_d;
   tievm_nicfsm_nfc_sys_addr_o    <= nfc_sys_addr;
    nicfsm_state                  <= nicfsm_nxt_state;
  end
end

assign tievm_nicfsm_nicfifo_rd_en_o  = nicfifo_rd_en;
assign tievm_nicfsm_nfc_wdata_valid_o= nfc_wdata_valid;
assign tievm_nicfsm_nicfifo_wr_en_o  = nicfifo_wr_en;

always @(*)
begin
  nfc_ce = 1;
  nfc_rd_req = 0;
  nfc_wr_req = 0;
//  nfc_wdata_valid = 0;
  nicfifo_wr_en = 0;
  nicfifo_rd_en = 0;

  delay_cnt_rst = 1;

  burst_cnt_d = 10'd0; 
 
  nfc_sys_addr = tievm_nicfsm_nfc_sys_addr_o;

  case(nicfsm_state)
  
  IDLE_STATE :  begin

                  nfc_sys_addr = 2'd0;

                  if(tievm_nic_nicfsm_wr_req_i | 
                     tievm_nic_nicfsm_rd_req_i  )
                  begin
                    if(tievm_nic_nicfsm_wr_req_i    &
                       tievm_nicfifo_nicfsm_fifo_full_i )
                    begin
                      nfc_ce = 0;
                      nicfifo_rd_en = 1;
                      nfc_wr_req = 1;
                      nicfsm_nxt_state = WRITE_STATE;
                    end
                    else if(tievm_nic_nicfsm_rd_req_i    &
                            !tievm_nicfifo_nicfsm_fifo_full_i)
                    begin
                      nfc_ce = 0;
                      nfc_rd_req = 1;
                      nicfsm_nxt_state = READ_STATE;
                    end
                    else
                    begin
                      nicfsm_nxt_state = IDLE_STATE;
                    end
                  end
                  else
                  begin
                    nicfsm_nxt_state = IDLE_STATE;
                  end

                end
 
  WRITE_STATE : begin

                  if(!tievm_nicfifo_nicfsm_fifo_empty_i)
                  begin 

                    if(!burst_cnt_up)
                    begin
                      
                      
                      
                      nfc_wr_req = 1;
                      nfc_ce = 0;
                      nicfsm_nxt_state = WRITE_STATE;

                      if(tievm_nfc_nicfsm_req_ack_i)
                      begin
                     //   nfc_wdata_valid = 1;
                        burst_cnt_d = burst_cnt + 10'd1;
                        nicfifo_rd_en = 1;
                      end
                      else
                      begin
                        burst_cnt_d = burst_cnt;
                      end

                    end
                    else
                    begin
                      nicfsm_nxt_state = WR_DELAY_STATE;
                    end

                  end
                  else
                  begin
                    nicfsm_nxt_state = NFC_BUSY;
                  end

                end


  WR_DELAY_STATE : begin
                      delay_cnt_rst = 0;
 
                    

                      if(delay_cnt_up_w)
                      begin
                        nicfsm_nxt_state = WRITE_STATE;
                        nicfifo_rd_en = 1;

                        nfc_sys_addr = tievm_nicfsm_nfc_sys_addr_o + 2'd1;
                      end
                      else
                      begin
                        nicfsm_nxt_state = WR_DELAY_STATE;
                        
                      end

                   end
                

  READ_STATE  : begin

                  if(!tievm_nicfifo_nicfsm_fifo_full_i)
                  begin
                    if(!burst_cnt_up)
                    begin

                      

                      nfc_rd_req = 1;
                      nfc_ce = 0;
                      nicfsm_nxt_state = READ_STATE;

                      if(tievm_nfc_nicfsm_req_ack_i)
                      begin
                        burst_cnt_d = burst_cnt + 10'd1;
                        nicfifo_wr_en = 1;
                      end
                      else
                      begin
                        burst_cnt_d = burst_cnt ;
                      end

                    end
                    else
                    begin
                      nicfifo_wr_en = 1;
                      nfc_rd_req = 1;
                      nicfsm_nxt_state = RD_DELAY_STATE;
                    end
                  end
                  else
                  begin
                    nicfsm_nxt_state = NFC_BUSY;
                  end

                end

  RD_DELAY_STATE : begin


                      delay_cnt_rst = 0;
                      
 
                      if(delay_cnt_up_w)
                      begin
                        nfc_rd_req = 0;
                        nicfsm_nxt_state = READ_STATE;
                        nfc_sys_addr = tievm_nicfsm_nfc_sys_addr_o + 2'd1;
                      end
                      else
                      begin
                        nfc_rd_req = 1;
                        nicfsm_nxt_state = RD_DELAY_STATE;
                      end

                   end

  NFC_BUSY    : begin
                 if(tievm_nic_nicfsm_task_done_i)
                 begin
                   nicfsm_nxt_state = IDLE_STATE;
                 end
                 else
                 begin
                   nicfsm_nxt_state = NFC_BUSY;
                 end
                end

  ERROR_STATE : begin
                 nicfsm_nxt_state = IDLE_STATE;
                end
     
  default     : begin
                 nicfsm_nxt_state = IDLE_STATE;
                end

  endcase


end



always @(negedge tievm_crm_nicfsm_clk_i or
         negedge tievm_crm_nicfsm_rst_ni
         )
begin
  if(!tievm_crm_nicfsm_rst_ni)
  begin
    nfc_wdata_valid <= 1'b0;
  end
  else 
  begin
    if((nicfsm_state == WRITE_STATE) &&
           tievm_nfc_nicfsm_req_ack_i   
           )
     begin
       nfc_wdata_valid <= 1'b1;
     end
     else if(burst_cnt_up_d)
     begin
        nfc_wdata_valid <= 1'b0;
     end
     else
     begin
        nfc_wdata_valid <= 1'b0;
     end
  end   
end
                                    
endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

