///////////////////////////////////////////////////////////////////////////////
// File name    : tievm_pmb_pmbfsm.v
// Project      : TI_C6457_LCEVM
// Created On   : 10/08/2010 (DD/MM/YYYY)
// Developers   : Christian Felix Y.
// Description  : This module implements the FSM for the interfacing with the 
//              : DSP GPIO
// Assumptions  : None
//              :
// Limitations  : None
//              :
// Known Errors : None
//              :
// Revision     : $Log: tievm_pmb_pmbfsm.v,v $
// Revision     : Revision 1.1  2010/09/17 11:23:09  amitkumar
// Revision     : added
// Revision     :
// Revision     : Revision 1.1  2010/09/17 06:42:35  christianfelix_rw
// Revision     : RELEASE RTL with RTL/BITFILE Version as 0x04.
// Revision     :
// Revision     : Revision 1.2  2010/09/06 08:25:11  christianfelix_rw
// Revision     : Copyright Added.
// Revision     :
// Revision     : Revision 1.1  2010/08/19 09:41:51  christianfelix_rw
// Revision     : Initial Version
// Revision     :
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module tievm_pmb_pmbfsm (
                         tievm_crm_pmbfsm_clk_i ,
                         tievm_crm_pmbfsm_rst_ni,
                         
                         //DSP-GPIO to FPGA Interface  
                         tievm_dsp_pmbfsm_cmd_i ,
                         tievm_dsp_pmbfsm_stb_i ,
                         tievm_pmbfsm_dsp_data_sel_o,  
                         tievm_pmbfsm_dsp_rdy_o ,
                                                        
                         //NIC Interface
                         tievm_pmbfsm_nic_wr_en_o,
                         tievm_pmbfsm_nic_rd_en_o,
                         tievm_nic_pmbfsm_rd_ack_i ,
                         tievm_nic_pmbfsm_wr_ack_i ,
                                                        
                         //CR Block interface
                         tievm_pmbfsm_cr_wr_en_o,
                         tievm_pmbfsm_cr_rd_en_o,
			 tievm_cr_pmbfsm_wr_ack_i,
			 tievm_cr_pmbfsm_rd_ack_i,
			 tievm_pmbfsm_cr_tout_err_o,


                                                        
                         //BMC Interface
                         tievm_bmc_pmbfsm_boot_done_i,
                                                           
                         //top module interface
                         tievm_pmbfsm_top_cmdreg_en_o,
                         tievm_pmbfsm_top_addreg_en_o,
                         tievm_pmbfsm_top_indatareg_en_o, 
                         tievm_pmbfsm_top_outdatareg_en_o,
                         
			 //PMB module interface
			 tievm_pmb_pmbfsm_fifo_sel_i
                         );

///////////////////////////////////////////////////////////////////////////////
//PARAMETERS
///////////////////////////////////////////////////////////////////////////////
parameter TIEVM_GPIO_CMD_WIDTH = 2;
parameter IDLE_STATE        = 3'b000;
parameter CMD_REG_STATE     = 3'b001;
parameter WRITE_STATE       = 3'b010;
parameter READ_STATE        = 3'b011;
parameter TX_COMPLETE_STATE = 3'b100;
parameter DELAY_STATE       = 3'b101;

parameter MST_TIMEOUT_COUNT = 19'h3FFFF;


///////////////////////////////////////////////////////////////////////////////
//INPUT OUTPUT PORTS
///////////////////////////////////////////////////////////////////////////////
input                                 tievm_crm_pmbfsm_clk_i ; 
input                                 tievm_crm_pmbfsm_rst_ni; 
input  [TIEVM_GPIO_CMD_WIDTH  - 1 : 0]tievm_dsp_pmbfsm_cmd_i ;
input                                 tievm_dsp_pmbfsm_stb_i ;
input                                 tievm_nic_pmbfsm_rd_ack_i ; 
input                                 tievm_nic_pmbfsm_wr_ack_i ; 
input                                 tievm_bmc_pmbfsm_boot_done_i;
input                                 tievm_pmb_pmbfsm_fifo_sel_i;
input                                 tievm_cr_pmbfsm_rd_ack_i;
input                                 tievm_cr_pmbfsm_wr_ack_i;


output                                tievm_pmbfsm_cr_wr_en_o; 
output                                tievm_pmbfsm_cr_rd_en_o; 
output                                tievm_pmbfsm_dsp_data_sel_o;  
output                                tievm_pmbfsm_dsp_rdy_o ;                            
output                                tievm_pmbfsm_nic_wr_en_o;                           
output                                tievm_pmbfsm_nic_rd_en_o;                           
output                                tievm_pmbfsm_top_cmdreg_en_o;     
output                                tievm_pmbfsm_top_addreg_en_o;     
output                                tievm_pmbfsm_top_indatareg_en_o;  
output                                tievm_pmbfsm_top_outdatareg_en_o; 

output                                tievm_pmbfsm_cr_tout_err_o;



///////////////////////////////////////////////////////////////////////////////
//OUTPUT PORT REGISTERS
///////////////////////////////////////////////////////////////////////////////
reg tievm_pmbfsm_cr_wr_en_o          ;
reg tievm_pmbfsm_cr_rd_en_o          ;
reg tievm_pmbfsm_dsp_data_sel_o      ; //sel = 0 -> boot mode parameters
                                       //sel = 1 -> CRB/NFM data to DSP
reg tievm_pmbfsm_dsp_rdy_o           ;                           
reg tievm_pmbfsm_nic_wr_en_o         ;                          
reg tievm_pmbfsm_nic_rd_en_o         ;                          
reg tievm_pmbfsm_top_addreg_en_o     ;    
reg tievm_pmbfsm_top_outdatareg_en_o ;

///////////////////////////////////////////////////////////////////////////////
//INTERNAL REGISTERS AND WIRES
///////////////////////////////////////////////////////////////////////////////
reg [2:0] pmbfsm_state;
reg [2:0] pmbfsm_nxt_state;

reg       cr_wr_en;
reg       cr_wr_en_d;
reg       cr_rd_en;
reg       dsp_data_sel; 
reg       dsp_rdy;                           
reg       nic_wr_en;                          
reg       nic_rd_en;                          
reg       cmdreg_en;    
reg       addreg_en;    
reg       indatareg_en; 
reg       outdatareg_en;
reg tievm_pmbfsm_dsp_rdy_d           ;                           

reg tout_count_en_cb;
reg tout_count_en ;

reg stb_d;
wire stb_h_to_l_w ; 
 
reg tout_count_up_d1; 
reg tout_count_up_d2;

wire tout_count_up_w ;



tievm_cntr #(.CNTR_WIDTH(19),
             .UP_COUNT(MST_TIMEOUT_COUNT)
	     )
mst_tout_cntr
       (
        .clk_i     (tievm_crm_pmbfsm_clk_i) ,                    
        .rst_n_i   (tievm_crm_pmbfsm_rst_ni) ,                    
        .sc_i      (tout_count_en) ,                          
        .count_o   () ,                        
	    .count_up_o(tout_count_up_w)
        );

assign tievm_pmbfsm_cr_tout_err_o = tout_count_up_d2; 
 
always @(posedge tievm_crm_pmbfsm_clk_i or 
         negedge tievm_crm_pmbfsm_rst_ni
	 )
begin
  if(!tievm_crm_pmbfsm_rst_ni)
  begin 
    tout_count_up_d1 <= 1'b0; 
    tout_count_up_d2 <= 1'b0; 
  end 
  else 
  begin 
    tout_count_up_d1 <=  tout_count_up_w; 
    tout_count_up_d2 <=  tout_count_up_d1; 
  end 
end 

///////////////////////////////////////////////////////////////////////////////
//Logic to detect the high to low edge of the input strobe.
///////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_pmbfsm_clk_i or 
         negedge tievm_crm_pmbfsm_rst_ni
	 )
begin
  if(!tievm_crm_pmbfsm_rst_ni)
  begin
    stb_d = 1'b0;
    cr_wr_en_d = 1'b0;
  end
  else
  begin
    stb_d = tievm_dsp_pmbfsm_stb_i ;
    cr_wr_en_d = cr_wr_en;
  end
end  


assign stb_h_to_l_w = stb_d & !tievm_dsp_pmbfsm_stb_i ;


///////////////////////////////////////////////////////////////////////////////
//Below code implements the basic FSM 
///////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_pmbfsm_clk_i or 
         negedge tievm_crm_pmbfsm_rst_ni
	 )
begin
  if(!tievm_crm_pmbfsm_rst_ni)
  begin
    pmbfsm_state                     <= 3'b000;
    tievm_pmbfsm_cr_wr_en_o          <= 1'b0;
    tievm_pmbfsm_cr_rd_en_o          <= 1'b0;
    tievm_pmbfsm_dsp_data_sel_o      <= 1'b0; //sel = 0 -> boot mode parameters
                                              //sel = 1 -> CRB/NFM data to DSP
    tievm_pmbfsm_dsp_rdy_d           <= 1'b1;                           
    tievm_pmbfsm_nic_wr_en_o         <= 1'b0;                          
    tievm_pmbfsm_nic_rd_en_o         <= 1'b0;                          
    tievm_pmbfsm_top_addreg_en_o     <= 1'b0;    
    tievm_pmbfsm_top_outdatareg_en_o <= 1'b0;

    tout_count_en                    <= 1'b1;
  end
  else
  begin
    pmbfsm_state                     <= pmbfsm_nxt_state;
    tievm_pmbfsm_cr_wr_en_o          <= (!cr_wr_en_d  &
                                          cr_wr_en);//for write just give a pulse
    tievm_pmbfsm_cr_rd_en_o          <= cr_rd_en;
    tievm_pmbfsm_dsp_data_sel_o      <= dsp_data_sel; 
    tievm_pmbfsm_dsp_rdy_d           <= dsp_rdy;                           
    tievm_pmbfsm_nic_wr_en_o         <= nic_wr_en;                          
    tievm_pmbfsm_nic_rd_en_o         <= nic_rd_en;                          
    tievm_pmbfsm_top_addreg_en_o     <= addreg_en;    
    tievm_pmbfsm_top_outdatareg_en_o <= outdatareg_en;

    tout_count_en                    <= tout_count_en_cb;
  end
end

assign tievm_pmbfsm_top_cmdreg_en_o = cmdreg_en ;
assign tievm_pmbfsm_top_indatareg_en_o = indatareg_en ;

always @(*)
begin

  nic_wr_en = 0;
  nic_rd_en = 0;
  cr_wr_en = 0;          	      
  cr_rd_en = 0;          	      
  cmdreg_en = 0;                       
  addreg_en = 0;         	      
  indatareg_en = 0;      
  outdatareg_en = 0;
  dsp_data_sel = 1; 

  tout_count_en_cb = tout_count_en ;

  case(pmbfsm_state)

  IDLE_STATE       :begin

                       if( tievm_bmc_pmbfsm_boot_done_i )
		       begin
                          
			 dsp_data_sel = 1;
                         outdatareg_en = 0;

		         if(tievm_dsp_pmbfsm_stb_i)
		         begin
		           dsp_rdy          = 0;
			   cmdreg_en        = 1;
		           pmbfsm_nxt_state = CMD_REG_STATE;
		         end
		         else//if(tievm_dsp_pmbfsm_stb_i)
		         begin
		           dsp_rdy = 1;
			   cmdreg_en = 0;
		           pmbfsm_nxt_state = IDLE_STATE;
		         end//else if(!tievm_dsp_pmbfsm_stb_i)

		       end
		       else
		       begin

		         dsp_rdy      = 1;
		         dsp_data_sel = 0;
                         outdatareg_en = 1;
			 cmdreg_en = 0;
		         pmbfsm_nxt_state = IDLE_STATE;
			 
		       end

                     end

  CMD_REG_STATE    :begin

                      dsp_rdy = 0;


                      if(tievm_dsp_pmbfsm_cmd_i[1:0] == 2'b01)//Write address
		      begin
                        addreg_en = 1;
			indatareg_en = 0;
			nic_rd_en = 0;
		        cr_rd_en = 0;
                        pmbfsm_nxt_state = DELAY_STATE;

                        tout_count_en_cb = 1'b1;
		      end
		      else if (tievm_dsp_pmbfsm_cmd_i[1:0] == 2'b11)//Write data
		      begin
		        addreg_en = 0;
			indatareg_en = 1;
			nic_wr_en = 0;
		        cr_wr_en = 0;
			pmbfsm_nxt_state = WRITE_STATE;

                        tout_count_en_cb = 1'b0;

		      end
		      else if (tievm_dsp_pmbfsm_cmd_i[1:0] == 2'b10)//Read data
		      begin
		        addreg_en = 0;
			indatareg_en = 0;
		        nic_rd_en = tievm_pmb_pmbfsm_fifo_sel_i;
		        cr_rd_en =  ! tievm_pmb_pmbfsm_fifo_sel_i;
		        pmbfsm_nxt_state = READ_STATE;

                        tout_count_en_cb = 1'b0;

		      end
		      else//NOP
		      begin
                        addreg_en = 0;
			indatareg_en = 0;
		        nic_rd_en = 0;
		        cr_rd_en =  0;
		        pmbfsm_nxt_state = DELAY_STATE;//editd on 3SEP09

                        tout_count_en_cb = 1'b1;
		      end

                    end

  WRITE_STATE      :begin

                      if(tievm_pmb_pmbfsm_fifo_sel_i)
		      begin
		        cr_wr_en = 0;

                        if(tievm_nic_pmbfsm_wr_ack_i |
			   tout_count_up_d2
			   )
		        begin
		          dsp_rdy = 1;
		          nic_wr_en = 0;
		          pmbfsm_nxt_state = DELAY_STATE;
		        end
		        else
		        begin
		          dsp_rdy = 0;
		          nic_wr_en = 1;
		          pmbfsm_nxt_state = WRITE_STATE;
		        end
		      end
		      else
		      begin

		        nic_wr_en = 0;

                        if(tievm_cr_pmbfsm_wr_ack_i |
			   tout_count_up_d2
			   )
		        begin
		          dsp_rdy = 1;
		          cr_wr_en = 0;
		          pmbfsm_nxt_state = DELAY_STATE;
		        end
		        else
		        begin
		          dsp_rdy = 0;
		          cr_wr_en = 1;
		          pmbfsm_nxt_state = WRITE_STATE;
		        end

		      end
                    end
  
  READ_STATE       :begin

                      if(tievm_pmb_pmbfsm_fifo_sel_i)
		      begin
                        
			cr_rd_en = 0;

                        if(tievm_nic_pmbfsm_rd_ack_i |
			   tout_count_up_d2
			   )
		        begin
		          dsp_rdy = 1;
		          nic_rd_en = 0;
		          outdatareg_en = 1;
		          pmbfsm_nxt_state = DELAY_STATE;
		        end
		        else
		        begin
		          dsp_rdy = 0;
		          nic_rd_en = 1;
		          outdatareg_en = 0;
		          pmbfsm_nxt_state = READ_STATE;
		        end

		      end
		      else
		      begin

		        nic_rd_en = 0;

                        if(tievm_cr_pmbfsm_rd_ack_i |
			   tout_count_up_d2
			   )
		        begin
		          dsp_rdy = 1;
		          cr_rd_en = 0;
		          outdatareg_en = 1;
		          pmbfsm_nxt_state = DELAY_STATE;
		        end
		        else
		        begin
		          dsp_rdy = 0;
		          cr_rd_en = 1;
		          outdatareg_en = 0;
		          pmbfsm_nxt_state = READ_STATE;
		        end

		      end
                    end


  DELAY_STATE      :begin

                      dsp_rdy = 1;

		      tout_count_en_cb = 1'b1;

		      if(!stb_h_to_l_w)
		      begin
                        outdatareg_en = tievm_pmbfsm_top_outdatareg_en_o;
		        pmbfsm_nxt_state = DELAY_STATE;
		      end
		      else
		      begin
			outdatareg_en = 0;
		        pmbfsm_nxt_state = IDLE_STATE;
		      end


                    end

  default           :begin
                      dsp_rdy = 1;
		      pmbfsm_nxt_state = IDLE_STATE;
                     end


  endcase
end

///////////////////////////////////////////////////////////////////////////////
//logic to delay the ready signal by 1 clock so that during the read operation
//data is first placed on the bus and then ready signal goes high
///////////////////////////////////////////////////////////////////////////////
always @(posedge tievm_crm_pmbfsm_clk_i or 
         negedge tievm_crm_pmbfsm_rst_ni
	 )
begin
  if(!tievm_crm_pmbfsm_rst_ni)
  begin
    tievm_pmbfsm_dsp_rdy_o <= 1'b0;
  end
  else
  begin
    tievm_pmbfsm_dsp_rdy_o <= tievm_pmbfsm_dsp_rdy_d ;
  end
end  

endmodule

///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010 Texas Instruments Incorporated - http://www.ti.com
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions 
//  are met:
//
//    Redistributions of source code must retain the above copyright 
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the 
//    documentation and/or other materials provided with the   
//    distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
///////////////////////////////////////////////////////////////////////////////

